@@ -9,15 +9,15 @@ __docformat__ = 'restructuredtext en'
 import os
 from functools import partial
 
-from PyQt5.Qt import (QComboBox, QLabel, QSpinBox, QDoubleSpinBox,
+from PyQt5.Qt import (Qt, QComboBox, QLabel, QSpinBox, QDoubleSpinBox,
         QDateTime, QGroupBox, QVBoxLayout, QSizePolicy, QGridLayout, QUrl,
         QSpacerItem, QIcon, QCheckBox, QWidget, QHBoxLayout, QLineEdit,
-        QPushButton, QMessageBox, QToolButton, QPlainTextEdit)
+        QMessageBox, QToolButton, QPlainTextEdit, QApplication, QStyle, QDialog)
 
 from calibre.utils.date import qt_to_dt, now, as_local_time, as_utc, internal_iso_format_string
 from calibre.gui2.complete2 import EditWithComplete
 from calibre.gui2.comments_editor import Editor as CommentsEditor
-from calibre.gui2 import UNDEFINED_QDATETIME, error_dialog
+from calibre.gui2 import UNDEFINED_QDATETIME, error_dialog, elided_text
 from calibre.gui2.dialogs.tag_editor import TagEditor
 from calibre.utils.config import tweaks
 from calibre.utils.icu import sort_key
@@ -34,6 +34,24 @@ def safe_disconnect(signal):
         pass
 
 
+def label_string(txt):
+    if txt:
+        try:
+            if txt[0].isalnum():
+                return '&' + txt
+        except:
+            pass
+    return txt
+
+
+def get_tooltip(col_metadata, add_index=False):
+    key = col_metadata['label'] + ('_index' if add_index else '')
+    label = col_metadata['name'] + (_(' index') if add_index else '')
+    description = col_metadata.get('display', {}).get('description', '')
+    return '{0} (#{1}){2} {3}'.format(
+                  label, key, ':' if description else '', description).strip()
+
+
 class Base(object):
 
     def __init__(self, db, col_id, parent=None):
@@ -42,21 +60,31 @@ class Base(object):
         self.initial_val = self.widgets = None
         self.signals_to_disconnect = []
         self.setup_ui(parent)
-        key = db.field_metadata.label_to_key(self.col_metadata['label'],
-                                                       prefer_custom=True)
-        description = self.col_metadata.get('display', {}).get('description', '')
-        if description:
-            description = key + ': ' + description
-        else:
-            description = key
+        description = get_tooltip(self.col_metadata)
         try:
+            self.widgets[0].setToolTip(description)
             self.widgets[1].setToolTip(description)
         except:
             try:
-                self.widgets[0].setToolTip(description)
+                self.widgets[1].setToolTip(description)
             except:
                 pass
 
+    def finish_ui_setup(self, parent, edit_widget):
+        self.was_none = False
+        w = QWidget(parent)
+        self.widgets.append(w)
+        l = QHBoxLayout()
+        l.setContentsMargins(0, 0, 0, 0)
+        w.setLayout(l)
+        self.editor = editor = edit_widget(parent)
+        l.addWidget(editor)
+        self.clear_button = QToolButton(parent)
+        self.clear_button.setIcon(QIcon(I('trash.png')))
+        self.clear_button.clicked.connect(self.set_to_undefined)
+        self.clear_button.setToolTip(_('Clear {0}').format(self.col_metadata['name']))
+        l.addWidget(self.clear_button)
+
     def initialize(self, book_id):
         val = self.db.get_custom(book_id, num=self.col_id, index_is_id=True)
         val = self.normalize_db_val(val)
@@ -101,27 +129,30 @@ class Base(object):
 class SimpleText(Base):
 
     def setup_ui(self, parent):
-        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent), QLineEdit(parent)]
+        self.editor = QLineEdit(parent)
+        self.widgets = [QLabel(label_string(self.col_metadata['name']), parent),
+                        self.editor]
+        self.editor.setClearButtonEnabled(True)
 
     def setter(self, val):
-        self.widgets[1].setText(unicode_type(val or ''))
+        self.editor.setText(unicode_type(val or ''))
 
     def getter(self):
-        return self.widgets[1].text().strip()
+        return self.editor.text().strip()
 
     def connect_data_changed(self, slot):
-        self.widgets[1].textChanged.connect(slot)
-        self.signals_to_disconnect.append(self.widgets[1].textChanged)
+        self.editor.textChanged.connect(slot)
+        self.signals_to_disconnect.append(self.editor.textChanged)
 
 
 class LongText(Base):
 
     def setup_ui(self, parent):
         self._box = QGroupBox(parent)
-        self._box.setTitle('&'+self.col_metadata['name'])
+        self._box.setTitle(label_string(self.col_metadata['name']))
         self._layout = QVBoxLayout()
         self._tb = QPlainTextEdit(self._box)
-        self._tb.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
+        self._tb.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)
         self._layout.addWidget(self._tb)
         self._box.setLayout(self._layout)
         self.widgets = [self._box]
@@ -140,7 +171,8 @@ class LongText(Base):
 class Bool(Base):
 
     def setup_ui(self, parent):
-        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent)]
+        name = self.col_metadata['name']
+        self.widgets = [QLabel(label_string(name), parent)]
         w = QWidget(parent)
         self.widgets.append(w)
 
@@ -150,32 +182,25 @@ class Bool(Base):
         self.combobox = QComboBox(parent)
         l.addWidget(self.combobox)
 
-        t = _('Yes')
-        c = QPushButton(t, parent)
-        width = c.fontMetrics().boundingRect(t).width() + 7
-        c.setMaximumWidth(width)
+        c = QToolButton(parent)
+        c.setIcon(QIcon(I('ok.png')))
+        c.setToolTip(_('Set {} to yes').format(name))
         l.addWidget(c)
         c.clicked.connect(self.set_to_yes)
 
-        t = _('No')
-        c = QPushButton(t, parent)
-        width = c.fontMetrics().boundingRect(t).width() + 7
-        c.setMaximumWidth(width)
+        c = QToolButton(parent)
+        c.setIcon(QIcon(I('list_remove.png')))
+        c.setToolTip(_('Set {} to no').format(name))
         l.addWidget(c)
         c.clicked.connect(self.set_to_no)
 
         if self.db.new_api.pref('bools_are_tristate'):
-            t = _('Clear')
-            c = QPushButton(t, parent)
-            width = c.fontMetrics().boundingRect(t).width() + 7
-            c.setMaximumWidth(width)
+            c = QToolButton(parent)
+            c.setIcon(QIcon(I('trash.png')))
+            c.setToolTip(_('Clear {}').format(name))
             l.addWidget(c)
             c.clicked.connect(self.set_to_cleared)
 
-        c = QLabel('', parent)
-        c.setMaximumWidth(1)
-        l.addWidget(c, 1)
-
         w = self.combobox
         items = [_('Yes'), _('No'), _('Undefined')]
         icons = [I('ok.png'), I('list_remove.png'), I('blank.png')]
@@ -212,67 +237,70 @@ class Bool(Base):
 class Int(Base):
 
     def setup_ui(self, parent):
-        self.was_none = False
-        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent),
-                ClearingSpinBox(parent)]
-        w = self.widgets[1]
-        w.setRange(-1000000, 100000000)
-        w.setSpecialValueText(_('Undefined'))
-        w.setSingleStep(1)
-        w.valueChanged.connect(self.valueChanged)
+        self.widgets = [QLabel(label_string(self.col_metadata['name']), parent)]
+        self.finish_ui_setup(parent, ClearingSpinBox)
+        self.editor.setRange(-1000000, 100000000)
+
+    def finish_ui_setup(self, parent, edit_widget):
+        Base.finish_ui_setup(self, parent, edit_widget)
+        self.editor.setSpecialValueText(_('Undefined'))
+        self.editor.setSingleStep(1)
+        self.editor.valueChanged.connect(self.valueChanged)
 
     def setter(self, val):
         if val is None:
-            val = self.widgets[1].minimum()
-        self.widgets[1].setValue(val)
-        self.was_none = val == self.widgets[1].minimum()
+            val = self.editor.minimum()
+        self.editor.setValue(val)
+        self.was_none = val == self.editor.minimum()
 
     def getter(self):
-        val = self.widgets[1].value()
-        if val == self.widgets[1].minimum():
+        val = self.editor.value()
+        if val == self.editor.minimum():
             val = None
         return val
 
     def valueChanged(self, to_what):
         if self.was_none and to_what == -999999:
             self.setter(0)
-        self.was_none = to_what == self.widgets[1].minimum()
+        self.was_none = to_what == self.editor.minimum()
 
     def connect_data_changed(self, slot):
-        self.widgets[1].valueChanged.connect(slot)
-        self.signals_to_disconnect.append(self.widgets[1].valueChanged)
+        self.editor.valueChanged.connect(slot)
+        self.signals_to_disconnect.append(self.editor.valueChanged)
+
+    def set_to_undefined(self):
+        self.editor.setValue(-1000000)
 
 
 class Float(Int):
 
     def setup_ui(self, parent):
-        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent),
-                ClearingDoubleSpinBox(parent)]
-        w = self.widgets[1]
-        w.setRange(-1000000., float(100000000))
-        w.setDecimals(2)
-        w.setSpecialValueText(_('Undefined'))
-        w.setSingleStep(1)
-        self.was_none = False
-        w.valueChanged.connect(self.valueChanged)
+        self.widgets = [QLabel(label_string(self.col_metadata['name']), parent)]
+        self.finish_ui_setup(parent, ClearingDoubleSpinBox)
+        self.editor.setRange(-1000000., float(100000000))
+        self.editor.setDecimals(2)
 
 
 class Rating(Base):
 
     def setup_ui(self, parent):
         allow_half_stars = self.col_metadata['display'].get('allow_half_stars', False)
-        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent), RatingEditor(parent=parent, is_half_star=allow_half_stars)]
+        self.widgets = [QLabel(label_string(self.col_metadata['name']), parent)]
+        self.finish_ui_setup(parent, partial(RatingEditor, is_half_star=allow_half_stars))
+
+    def set_to_undefined(self):
+        self.editor.setCurrentIndex(0)
 
     def setter(self, val):
         val = max(0, min(int(val or 0), 10))
-        self.widgets[1].rating_value = val
+        self.editor.rating_value = val
 
     def getter(self):
-        return self.widgets[1].rating_value or None
+        return self.editor.rating_value or None
 
     def connect_data_changed(self, slot):
-        self.widgets[1].currentTextChanged.connect(slot)
-        self.signals_to_disconnect.append(self.widgets[1].currentTextChanged)
+        self.editor.currentTextChanged.connect(slot)
+        self.signals_to_disconnect.append(self.editor.currentTextChanged)
 
 
 class DateTimeEdit(DateTimeEditBase):
@@ -297,42 +325,44 @@ class DateTime(Base):
 
     def setup_ui(self, parent):
         cm = self.col_metadata
-        self.widgets = [QLabel('&'+cm['name']+':', parent), DateTimeEdit(parent)]
-        self.widgets.append(QLabel(''))
+        self.widgets = [QLabel(label_string(cm['name']), parent)]
         w = QWidget(parent)
         self.widgets.append(w)
         l = QHBoxLayout()
         l.setContentsMargins(0, 0, 0, 0)
         w.setLayout(l)
-        l.addStretch(1)
-        self.today_button = QPushButton(_('Set \'%s\' to today')%cm['name'], parent)
-        l.addWidget(self.today_button)
-        self.clear_button = QPushButton(_('Clear \'%s\'')%cm['name'], parent)
-        l.addWidget(self.clear_button)
-        l.addStretch(2)
-
-        w = self.widgets[1]
+        self.dte = dte = DateTimeEdit(parent)
         format_ = cm['display'].get('date_format','')
         if not format_:
             format_ = 'dd MMM yyyy hh:mm'
         elif format_ == 'iso':
             format_ = internal_iso_format_string()
-        w.setDisplayFormat(format_)
-        w.setCalendarPopup(True)
-        w.setMinimumDateTime(UNDEFINED_QDATETIME)
-        w.setSpecialValueText(_('Undefined'))
-        self.today_button.clicked.connect(w.set_to_today)
-        self.clear_button.clicked.connect(w.set_to_clear)
+        dte.setDisplayFormat(format_)
+        dte.setCalendarPopup(True)
+        dte.setMinimumDateTime(UNDEFINED_QDATETIME)
+        dte.setSpecialValueText(_('Undefined'))
+        l.addWidget(dte)
+
+        self.today_button = QToolButton(parent)
+        self.today_button.setText(_('Today'))
+        self.today_button.clicked.connect(dte.set_to_today)
+        l.addWidget(self.today_button)
+
+        self.clear_button = QToolButton(parent)
+        self.clear_button.setIcon(QIcon(I('trash.png')))
+        self.clear_button.clicked.connect(dte.set_to_clear)
+        self.clear_button.setToolTip(_('Clear {0}').format(self.col_metadata['name']))
+        l.addWidget(self.clear_button)
 
     def setter(self, val):
         if val is None:
-            val = self.widgets[1].minimumDateTime()
+            val = self.dte.minimumDateTime()
         else:
             val = QDateTime(val)
-        self.widgets[1].setDateTime(val)
+        self.dte.setDateTime(val)
 
     def getter(self):
-        val = self.widgets[1].dateTime()
+        val = self.dte.dateTime()
         if val <= UNDEFINED_QDATETIME:
             val = None
         else:
@@ -346,18 +376,18 @@ class DateTime(Base):
         return as_utc(val) if val is not None else None
 
     def connect_data_changed(self, slot):
-        self.widgets[1].dateTimeChanged.connect(slot)
-        self.signals_to_disconnect.append(self.widgets[1].dateTimeChanged)
+        self.dte.dateTimeChanged.connect(slot)
+        self.signals_to_disconnect.append(self.dte.dateTimeChanged)
 
 
 class Comments(Base):
 
     def setup_ui(self, parent):
         self._box = QGroupBox(parent)
-        self._box.setTitle('&'+self.col_metadata['name'])
+        self._box.setTitle(label_string(self.col_metadata['name']))
         self._layout = QVBoxLayout()
         self._tb = CommentsEditor(self._box, toolbar_prefs_name='metadata-comments-editor-widget-hidden-toolbars')
-        self._tb.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
+        self._tb.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)
         # self._tb.setTabChangesFocus(True)
         self._layout.addWidget(self._tb)
         self._box.setLayout(self._layout)
@@ -450,7 +480,7 @@ def _save_dialog(parent, title, msg, det_msg=''):
     d = QMessageBox(parent)
     d.setWindowTitle(title)
     d.setText(msg)
-    d.setStandardButtons(QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
+    d.setStandardButtons(QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No | QMessageBox.StandardButton.Cancel)
     return d.exec_()
 
 
@@ -469,20 +499,23 @@ class Text(Base):
                 w.set_space_before_sep(True)
                 w.set_add_separator(tweaks['authors_completer_append_separator'])
             w.get_editor_button().clicked.connect(self.edit)
-            w.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
+            w.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Preferred)
+            self.set_to_undefined = w.clear
         else:
             w = EditWithComplete(parent)
             w.set_separator(None)
             w.setSizeAdjustPolicy(w.AdjustToMinimumContentsLengthWithIcon)
             w.setMinimumContentsLength(25)
-        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent), w]
+            self.set_to_undefined = w.clearEditText
+        self.widgets = [QLabel(label_string(self.col_metadata['name']), parent)]
+        self.finish_ui_setup(parent, lambda parent: w)
 
     def initialize(self, book_id):
         values = list(self.db.all_custom(num=self.col_id))
         values.sort(key=sort_key)
         self.book_id = book_id
-        self.widgets[1].clear()
-        self.widgets[1].update_items_cache(values)
+        self.editor.clear()
+        self.editor.update_items_cache(values)
         val = self.db.get_custom(book_id, num=self.col_id, index_is_id=True)
         if isinstance(val, list):
             if not self.col_metadata.get('display', {}).get('is_names', False):
@@ -492,23 +525,23 @@ class Text(Base):
         if self.col_metadata['is_multiple']:
             self.setter(val)
         else:
-            self.widgets[1].setText(val)
+            self.editor.setText(val)
         self.initial_val = self.current_val
 
     def setter(self, val):
         if self.col_metadata['is_multiple']:
             if not val:
                 val = []
-            self.widgets[1].setText(self.sep['list_to_ui'].join(val))
+            self.editor.setText(self.sep['list_to_ui'].join(val))
 
     def getter(self):
         if self.col_metadata['is_multiple']:
-            val = unicode_type(self.widgets[1].text()).strip()
+            val = unicode_type(self.editor.text()).strip()
             ans = [x.strip() for x in val.split(self.sep['ui_to_list']) if x.strip()]
             if not ans:
                 ans = None
             return ans
-        val = unicode_type(self.widgets[1].currentText()).strip()
+        val = unicode_type(self.editor.currentText()).strip()
         if not val:
             val = None
         return val
@@ -519,23 +552,23 @@ class Text(Base):
                     _('You have changed the values. In order to use this '
                        'editor, you must either discard or apply these '
                        'changes. Apply changes?'))
-            if d == QMessageBox.Cancel:
+            if d == QMessageBox.StandardButton.Cancel:
                 return
-            if d == QMessageBox.Yes:
+            if d == QMessageBox.StandardButton.Yes:
                 self.commit(self.book_id)
                 self.db.commit()
                 self.initial_val = self.current_val
             else:
                 self.setter(self.initial_val)
         d = TagEditor(self.parent, self.db, self.book_id, self.key)
-        if d.exec_() == TagEditor.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             self.setter(d.tags)
 
     def connect_data_changed(self, slot):
         if self.col_metadata['is_multiple']:
-            s = self.widgets[1].tags_box.currentTextChanged
+            s = self.editor.tags_box.currentTextChanged
         else:
-            s = self.widgets[1].currentTextChanged
+            s = self.editor.currentTextChanged
         s.connect(slot)
         self.signals_to_disconnect.append(s)
 
@@ -548,17 +581,25 @@ class Series(Base):
         w.setSizeAdjustPolicy(w.AdjustToMinimumContentsLengthWithIcon)
         w.setMinimumContentsLength(25)
         self.name_widget = w
-        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent), w]
+        self.widgets = [QLabel(label_string(self.col_metadata['name']), parent)]
+        self.finish_ui_setup(parent, lambda parent: w)
         w.editTextChanged.connect(self.series_changed)
 
-        self.widgets.append(QLabel('&'+self.col_metadata['name']+_(' index:'), parent))
+        w = QLabel(label_string(self.col_metadata['name'])+_(' index'), parent)
+        w.setToolTip(get_tooltip(self.col_metadata, add_index=True))
+        self.widgets.append(w)
         w = QDoubleSpinBox(parent)
         w.setRange(-10000., float(100000000))
         w.setDecimals(2)
         w.setSingleStep(1)
         self.idx_widget=w
+        w.setToolTip(get_tooltip(self.col_metadata, add_index=True))
         self.widgets.append(w)
 
+    def set_to_undefined(self):
+        self.name_widget.clearEditText()
+        self.idx_widget.setValue(1.0)
+
     def initialize(self, book_id):
         values = list(self.db.all_custom(num=self.col_id))
         values.sort(key=sort_key)
@@ -613,7 +654,7 @@ class Series(Base):
         mi.set('#' + self.col_metadata['label'], val, extra=s_index)
 
     def connect_data_changed(self, slot):
-        for s in self.widgets[1].editTextChanged, self.widgets[3].valueChanged:
+        for s in self.name_widget.editTextChanged, self.idx_widget.valueChanged:
             s.connect(slot)
             self.signals_to_disconnect.append(s)
 
@@ -622,18 +663,17 @@ class Enumeration(Base):
 
     def setup_ui(self, parent):
         self.parent = parent
-        self.widgets = [QLabel('&'+self.col_metadata['name']+':', parent),
-                QComboBox(parent)]
-        w = self.widgets[1]
+        self.widgets = [QLabel(label_string(self.col_metadata['name']), parent)]
+        self.finish_ui_setup(parent, QComboBox)
         vals = self.col_metadata['display']['enum_values']
-        w.addItem('')
+        self.editor.addItem('')
         for v in vals:
-            w.addItem(v)
+            self.editor.addItem(v)
 
     def initialize(self, book_id):
         val = self.db.get_custom(book_id, num=self.col_id, index_is_id=True)
         val = self.normalize_db_val(val)
-        idx = self.widgets[1].findText(val)
+        idx = self.editor.findText(val)
         if idx < 0:
             error_dialog(self.parent, '',
                     _('The enumeration "{0}" contains an invalid value '
@@ -642,14 +682,14 @@ class Enumeration(Base):
                     show=True, show_copy_button=False)
 
             idx = 0
-        self.widgets[1].setCurrentIndex(idx)
+        self.editor.setCurrentIndex(idx)
         self.initial_val = self.current_val
 
     def setter(self, val):
-        self.widgets[1].setCurrentIndex(self.widgets[1].findText(val))
+        self.editor.setCurrentIndex(self.editor.findText(val))
 
     def getter(self):
-        return unicode_type(self.widgets[1].currentText())
+        return unicode_type(self.editor.currentText())
 
     def normalize_db_val(self, val):
         if val is None:
@@ -661,9 +701,12 @@ class Enumeration(Base):
             val = None
         return val
 
+    def set_to_undefined(self):
+        self.editor.setCurrentIndex(0)
+
     def connect_data_changed(self, slot):
-        self.widgets[1].currentIndexChanged.connect(slot)
-        self.signals_to_disconnect.append(self.widgets[1].currentIndexChanged)
+        self.editor.currentIndexChanged.connect(slot)
+        self.signals_to_disconnect.append(self.editor.currentIndexChanged)
 
 
 def comments_factory(db, key, parent):
@@ -743,10 +786,16 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
         turnover_point = count + 1000
     ans = []
     column = row = base_row = max_row = 0
-    minimum_label = 0
+    label_width = 0
+    do_elision = tweaks['metadata_edit_elide_labels']
+    elide_pos = tweaks['metadata_edit_elision_point']
+    elide_pos = elide_pos if elide_pos in {'left', 'middle', 'right'} else 'right'
+    # make room on the right side for the scrollbar
+    sb_width = QApplication.instance().style().pixelMetric(QStyle.PixelMetric.PM_ScrollBarExtent)
+    layout.setContentsMargins(0, 0, sb_width, 0)
     for key in cols:
         if not fm[key]['is_editable']:
-            continue  # this almost never happens
+            continue  # The job spy plugin can change is_editable
         dt = fm[key]['datatype']
         if dt == 'composite' or (bulk and dt == 'comments'):
             continue
@@ -778,37 +827,36 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
             row += 1
         for c in range(0, len(w.widgets), 2):
             if not is_comments:
-                w.widgets[c].setWordWrap(True)
-                '''
-                It seems that there is something strange with wordwrapped labels
-                with some fonts. Apparently one part of QT thinks it is showing
-                a single line and sizes the line vertically accordingly. Another
-                part thinks there isn't enough space and wraps the label. The
-                result is two lines in a single line space, cutting off parts of
-                the lines. It doesn't happen with every font, nor with every
-                "long" label.
-
-                This change works around the problem by setting the maximum
-                display width and telling QT to respect that width.
-
-                While here I implemented an arbitrary minimum label length so
-                that there is a better chance that the field edit boxes line up.
-                '''
-                if minimum_label == 0:
-                    minimum_label = w.widgets[c].fontMetrics().boundingRect('smallLabel').width()
-                label_width = w.widgets[c].fontMetrics().boundingRect(w.widgets[c].text()).width()
+                # Set the label column width to a fixed size. Elide labels that
+                # don't fit
+                wij = w.widgets[c]
+                if label_width == 0:
+                    font_metrics = wij.fontMetrics()
+                    colon_width = font_metrics.width(':')
+                    if bulk:
+                        label_width = (font_metrics.averageCharWidth() *
+                               tweaks['metadata_edit_bulk_cc_label_length']) - colon_width
+                    else:
+                        label_width = (font_metrics.averageCharWidth() *
+                               tweaks['metadata_edit_single_cc_label_length']) - colon_width
+                wij.setMaximumWidth(label_width)
                 if c == 0:
-                    w.widgets[0].setMaximumWidth(label_width)
-                    w.widgets[0].setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Preferred)
-                    l.setColumnMinimumWidth(0, minimum_label)
-                else:
-                    w.widgets[0].setMaximumWidth(max(w.widgets[0].maximumWidth(), label_width))
-                w.widgets[c].setBuddy(w.widgets[c+1])
-                l.addWidget(w.widgets[c], c, 0)
+                    wij.setSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Preferred)
+                    l.setColumnMinimumWidth(0, label_width)
+                wij.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignmentFlag.AlignVCenter)
+                t = unicode_type(wij.text())
+                if t:
+                    if do_elision:
+                        wij.setText(elided_text(t, font=font_metrics,
+                                            width=label_width, pos=elide_pos) + ':')
+                    else:
+                        wij.setText(t + ':')
+                        wij.setWordWrap(True)
+                wij.setBuddy(w.widgets[c+1])
+                l.addWidget(wij, c, 0)
                 l.addWidget(w.widgets[c+1], c, 1)
             else:
                 l.addWidget(w.widgets[0], 0, 0, 1, 2)
-        l.addItem(QSpacerItem(0, 0, vPolicy=QSizePolicy.Expanding), c, 0, 1, 1)
         max_row = max(max_row, row)
         if row >= turnover_point:
             column = 1
@@ -817,8 +865,8 @@ def populate_metadata_page(layout, db, book_id, bulk=False, two_column=False, pa
 
     items = []
     if len(ans) > 0:
-        items.append(QSpacerItem(10, 10, QSizePolicy.Minimum,
-            QSizePolicy.Expanding))
+        items.append(QSpacerItem(10, 10, QSizePolicy.Policy.Minimum,
+            QSizePolicy.Policy.Expanding))
         layout.addItem(items[-1], layout.rowCount(), 0, 1, 1)
         layout.setRowStretch(layout.rowCount()-1, 100)
     return ans, items
@@ -848,6 +896,34 @@ class BulkBase(Base):
             ans = list(ans)
         return ans
 
+    def finish_ui_setup(self, parent, is_bool=False, add_edit_tags_button=(False,)):
+        self.was_none = False
+        l = self.widgets[1].layout()
+        if not is_bool or self.bools_are_tristate:
+            self.clear_button = QToolButton(parent)
+            self.clear_button.setIcon(QIcon(I('trash.png')))
+            self.clear_button.setToolTip(_('Clear {0}').format(self.col_metadata['name']))
+            self.clear_button.clicked.connect(self.set_to_undefined)
+            l.insertWidget(1, self.clear_button)
+        if is_bool:
+            self.set_no_button = QToolButton(parent)
+            self.set_no_button.setIcon(QIcon(I('list_remove.png')))
+            self.set_no_button.clicked.connect(lambda:self.main_widget.setCurrentIndex(1))
+            self.set_no_button.setToolTip(_('Set {0} to No').format(self.col_metadata['name']))
+            l.insertWidget(1, self.set_no_button)
+            self.set_yes_button = QToolButton(parent)
+            self.set_yes_button.setIcon(QIcon(I('ok.png')))
+            self.set_yes_button.clicked.connect(lambda:self.main_widget.setCurrentIndex(0))
+            self.set_yes_button.setToolTip(_('Set {0} to Yes').format(self.col_metadata['name']))
+            l.insertWidget(1, self.set_yes_button)
+        if add_edit_tags_button[0]:
+            self.edit_tags_button = QToolButton(parent)
+            self.edit_tags_button.setToolTip(_('Open item editor'))
+            self.edit_tags_button.setIcon(QIcon(I('chapters.png')))
+            self.edit_tags_button.clicked.connect(add_edit_tags_button[1])
+            l.insertWidget(1, self.edit_tags_button)
+        l.insertStretch(2)
+
     def initialize(self, book_ids):
         self.initial_val = val = self.get_initial_value(book_ids)
         val = self.normalize_db_val(val)
@@ -860,21 +936,15 @@ class BulkBase(Base):
         val = self.normalize_ui_val(val)
         self.db.set_custom_bulk(book_ids, val, num=self.col_id, notify=notify)
 
-    def make_widgets(self, parent, main_widget_class, extra_label_text='',
-                     add_tags_edit_button=False):
+    def make_widgets(self, parent, main_widget_class):
         w = QWidget(parent)
-        self.widgets = [QLabel('&'+self.col_metadata['name']+':', w), w]
+        self.widgets = [QLabel(label_string(self.col_metadata['name']), w), w]
         l = QHBoxLayout()
         l.setContentsMargins(0, 0, 0, 0)
         w.setLayout(l)
         self.main_widget = main_widget_class(w)
         l.addWidget(self.main_widget)
         l.setStretchFactor(self.main_widget, 10)
-        if add_tags_edit_button:
-            self.edit_tags_button = QToolButton(parent)
-            self.edit_tags_button.setToolTip(_('Open item editor'))
-            self.edit_tags_button.setIcon(QIcon(I('chapters.png')))
-            l.addWidget(self.edit_tags_button)
         self.a_c_checkbox = QCheckBox(_('Apply changes'), w)
         l.addWidget(self.a_c_checkbox)
         self.ignore_change_signals = True
@@ -918,7 +988,8 @@ class BulkBool(BulkBase, Bool):
     def setup_ui(self, parent):
         self.make_widgets(parent, QComboBox)
         items = [_('Yes'), _('No')]
-        if not self.db.new_api.pref('bools_are_tristate'):
+        self.bools_are_tristate = self.db.new_api.pref('bools_are_tristate')
+        if not self.bools_are_tristate:
             items.append('')
         else:
             items.append(_('Undefined'))
@@ -927,10 +998,15 @@ class BulkBool(BulkBase, Bool):
         for icon, text in zip(icons, items):
             self.main_widget.addItem(QIcon(icon), text)
         self.main_widget.blockSignals(False)
+        self.finish_ui_setup(parent, is_bool=True)
+
+    def set_to_undefined(self):
+        # Only called if bools are tristate
+        self.main_widget.setCurrentIndex(2)
 
     def getter(self):
         val = self.main_widget.currentIndex()
-        if not self.db.new_api.pref('bools_are_tristate'):
+        if not self.bools_are_tristate:
             return {2: False, 1: False, 0: True}[val]
         else:
             return {2: None, 1: False, 0: True}[val]
@@ -945,14 +1021,13 @@ class BulkBool(BulkBase, Bool):
             return
         val = self.gui_val
         val = self.normalize_ui_val(val)
-        if not self.db.new_api.pref('bools_are_tristate') and val is None:
+        if not self.bools_are_tristate and val is None:
             val = False
         self.db.set_custom_bulk(book_ids, val, num=self.col_id, notify=notify)
 
     def a_c_checkbox_changed(self):
         if not self.ignore_change_signals:
-            if not self.db.new_api.pref('bools_are_tristate') and \
-                                    self.main_widget.currentIndex() == 2:
+            if not self.bools_are_tristate and self.main_widget.currentIndex() == 2:
                 self.a_c_checkbox.setChecked(False)
             else:
                 self.a_c_checkbox.setChecked(True)
@@ -961,9 +1036,12 @@ class BulkBool(BulkBase, Bool):
 class BulkInt(BulkBase):
 
     def setup_ui(self, parent):
-        self.was_none = False
         self.make_widgets(parent, QSpinBox)
         self.main_widget.setRange(-1000000, 100000000)
+        self.finish_ui_setup(parent)
+
+    def finish_ui_setup(self, parent):
+        BulkBase.finish_ui_setup(self, parent)
         self.main_widget.setSpecialValueText(_('Undefined'))
         self.main_widget.setSingleStep(1)
         self.main_widget.valueChanged.connect(self.valueChanged)
@@ -986,6 +1064,9 @@ class BulkInt(BulkBase):
             self.setter(0)
         self.was_none = to_what == self.main_widget.minimum()
 
+    def set_to_undefined(self):
+        self.main_widget.setValue(-1000000)
+
 
 class BulkFloat(BulkInt):
 
@@ -993,10 +1074,10 @@ class BulkFloat(BulkInt):
         self.make_widgets(parent, QDoubleSpinBox)
         self.main_widget.setRange(-1000000., float(100000000))
         self.main_widget.setDecimals(2)
-        self.main_widget.setSpecialValueText(_('Undefined'))
-        self.main_widget.setSingleStep(1)
-        self.was_none = False
-        self.main_widget.valueChanged.connect(self.valueChanged)
+        self.finish_ui_setup(parent)
+
+    def set_to_undefined(self):
+        self.main_widget.setValue(-1000000.)
 
 
 class BulkRating(BulkBase):
@@ -1004,6 +1085,10 @@ class BulkRating(BulkBase):
     def setup_ui(self, parent):
         allow_half_stars = self.col_metadata['display'].get('allow_half_stars', False)
         self.make_widgets(parent, partial(RatingEditor, is_half_star=allow_half_stars))
+        self.finish_ui_setup(parent)
+
+    def set_to_undefined(self):
+        self.main_widget.setCurrentIndex(0)
 
     def setter(self, val):
         val = max(0, min(int(val or 0), 10))
@@ -1019,18 +1104,15 @@ class BulkDateTime(BulkBase):
     def setup_ui(self, parent):
         cm = self.col_metadata
         self.make_widgets(parent, DateTimeEdit)
-        self.widgets.append(QLabel(''))
-        w = QWidget(parent)
-        self.widgets.append(w)
-        l = QHBoxLayout()
-        l.setContentsMargins(0, 0, 0, 0)
-        w.setLayout(l)
-        l.addStretch(1)
-        self.today_button = QPushButton(_('Set \'%s\' to today')%cm['name'], parent)
-        l.addWidget(self.today_button)
-        self.clear_button = QPushButton(_('Clear \'%s\'')%cm['name'], parent)
-        l.addWidget(self.clear_button)
-        l.addStretch(2)
+        l = self.widgets[1].layout()
+        self.today_button = QToolButton(parent)
+        self.today_button.setText(_('Today'))
+        l.insertWidget(1, self.today_button)
+        self.clear_button = QToolButton(parent)
+        self.clear_button.setIcon(QIcon(I('trash.png')))
+        self.clear_button.setToolTip(_('Clear {0}').format(self.col_metadata['name']))
+        l.insertWidget(2, self.clear_button)
+        l.insertStretch(3)
 
         w = self.main_widget
         format_ = cm['display'].get('date_format','')
@@ -1115,7 +1197,7 @@ class BulkSeries(BulkBase):
             'force series numbers.') + '</p>')
         self.series_increment.setPrefix('+')
         layout.addWidget(self.series_increment)
-        layout.addItem(QSpacerItem(20, 10, QSizePolicy.Expanding, QSizePolicy.Minimum))
+        layout.addItem(QSpacerItem(20, 10, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum))
         self.widgets.append(w)
         self.idx_widget.stateChanged.connect(self.a_c_checkbox_changed)
         self.force_number.stateChanged.connect(self.a_c_checkbox_changed)
@@ -1234,12 +1316,16 @@ class BulkEnumeration(BulkBase, Enumeration):
     def setup_ui(self, parent):
         self.parent = parent
         self.make_widgets(parent, QComboBox)
+        self.finish_ui_setup(parent)
         vals = self.col_metadata['display']['enum_values']
         self.main_widget.blockSignals(True)
         self.main_widget.addItem('')
         self.main_widget.addItems(vals)
         self.main_widget.blockSignals(False)
 
+    def set_to_undefined(self):
+        self.main_widget.setCurrentIndex(0)
+
     def getter(self):
         return unicode_type(self.main_widget.currentText())
 
@@ -1285,19 +1371,22 @@ class BulkText(BulkBase):
     def setup_ui(self, parent):
         values = self.all_values = list(self.db.all_custom(num=self.col_id))
         values.sort(key=sort_key)
+        is_tags = False
         if self.col_metadata['is_multiple']:
             is_tags = not self.col_metadata['display'].get('is_names', False)
-            self.make_widgets(parent, EditWithComplete,
-                              extra_label_text=_('tags to add'),
-                              add_tags_edit_button=is_tags)
-            self.main_widget.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
+            self.make_widgets(parent, EditWithComplete)
+            self.main_widget.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Preferred)
             self.adding_widget = self.main_widget
 
             if is_tags:
-                self.edit_tags_button.clicked.connect(self.edit_add)
                 w = RemoveTags(parent, values)
                 w.remove_tags_button.clicked.connect(self.edit_remove)
-                self.widgets.append(QLabel('&'+self.col_metadata['name']+': ' + _('tags to remove'), parent))
+                l = QLabel(label_string(self.col_metadata['name'])+': ' +
+                                           _('tags to remove'), parent)
+                tt = get_tooltip(self.col_metadata) + ': ' + _('tags to remove')
+                l.setToolTip(tt)
+                self.widgets.append(l)
+                w.setToolTip(tt)
                 self.widgets.append(w)
                 self.removing_widget = w
                 self.main_widget.set_separator(',')
@@ -1316,6 +1405,10 @@ class BulkText(BulkBase):
             self.main_widget.setMinimumContentsLength(25)
         self.ignore_change_signals = False
         self.parent = parent
+        self.finish_ui_setup(parent, add_edit_tags_button=(is_tags,self.edit_add))
+
+    def set_to_undefined(self):
+        self.main_widget.clearEditText()
 
     def initialize(self, book_ids):
         self.main_widget.update_items_cache(self.all_values)
@@ -1382,11 +1475,11 @@ class BulkText(BulkBase):
                     _('You have entered values. In order to use this '
                        'editor you must first discard them. '
                        'Discard the values?'))
-            if d == QMessageBox.Cancel or d == QMessageBox.No:
+            if d == QMessageBox.StandardButton.Cancel or d == QMessageBox.StandardButton.No:
                 return
             widget.setText('')
         d = TagEditor(self.parent, self.db, key=('#'+self.col_metadata['label']))
-        if d.exec_() == TagEditor.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             val = d.tags
             if not val:
                 val = []
