@@ -11,7 +11,7 @@ from PyQt5.Qt import (
     QVBoxLayout, QStackedWidget, QSize, QPushButton, QIcon, QWidget, QListView,
     QHBoxLayout, QAbstractListModel, Qt, QLabel, QSizePolicy, pyqtSignal, QSortFilterProxyModel,
     QFormLayout, QSpinBox, QLineEdit, QGroupBox, QListWidget, QListWidgetItem,
-    QToolButton, QTreeView)
+    QToolButton, QTreeView, QDialog, QDialogButtonBox)
 
 from calibre.gui2 import error_dialog, open_local_file, choose_files, choose_save_file
 from calibre.gui2.dialogs.confirm_delete import confirm as confirm_delete
@@ -64,7 +64,7 @@ class CustomRecipeModel(QAbstractListModel):  # {{{
             return 0
 
     def data(self, index, role):
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             return self.title(index)
 
     def update(self, row, title, script):
@@ -175,7 +175,7 @@ class RecipeList(QWidget):  # {{{
         l.addWidget(v)
 
         self.stacks = s = QStackedWidget(self)
-        l.addWidget(s, stretch=10, alignment=Qt.AlignTop)
+        l.addWidget(s, stretch=10, alignment=Qt.AlignmentFlag.AlignTop)
 
         self.first_msg = la = QLabel(_(
             'Create a new news source by clicking one of the buttons below'))
@@ -188,25 +188,25 @@ class RecipeList(QWidget):  # {{{
         s.addWidget(w)
 
         self.title = la = QLabel(w)
-        la.setAlignment(Qt.AlignHCenter | Qt.AlignTop)
+        la.setAlignment(Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignTop)
         l.addWidget(la)
         l.setSpacing(20)
 
         self.edit_button = b = QPushButton(QIcon(I('modified.png')), _('&Edit this recipe'), w)
-        b.setSizePolicy(QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed))
+        b.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed))
         b.clicked.connect(self.edit_requested)
         l.addWidget(b)
         self.remove_button = b = QPushButton(QIcon(I('list_remove.png')), _('&Remove this recipe'), w)
         b.clicked.connect(self.remove)
-        b.setSizePolicy(QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed))
+        b.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed))
         l.addWidget(b)
         self.export_button = b = QPushButton(QIcon(I('save.png')), _('S&ave recipe as file'), w)
-        b.setSizePolicy(QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed))
+        b.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed))
         b.clicked.connect(self.save_recipe)
         l.addWidget(b)
         self.download_button = b = QPushButton(QIcon(I('download-metadata.png')), _('&Download this recipe'), w)
         b.clicked.connect(self.download)
-        b.setSizePolicy(QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed))
+        b.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed))
         l.addWidget(b)
 
         self.select_row()
@@ -394,7 +394,7 @@ class BasicRecipe(QWidget):  # {{{
         if not title:
             return error_dialog(self, _('No feed URL'), _(
                 'You must specify a URL for the feed'), show=True)
-        QListWidgetItem('%s - %s' % (title, url), self.feeds).setData(Qt.UserRole, (title, url))
+        QListWidgetItem('%s - %s' % (title, url), self.feeds).setData(Qt.ItemDataRole.UserRole, (title, url))
         self.feed_title.clear(), self.feed_url.clear()
 
     def validate(self):
@@ -419,7 +419,7 @@ class BasicRecipe(QWidget):  # {{{
     def recipe_source(self):
 
         title = self.title.text().strip()
-        feeds = [self.feeds.item(i).data(Qt.UserRole) for i in range(self.feeds.count())]
+        feeds = [self.feeds.item(i).data(Qt.ItemDataRole.UserRole) for i in range(self.feeds.count())]
         return options_to_recipe_source(title, self.oldest_article.value(), self.max_articles.value(), feeds)
 
     @recipe_source.setter
@@ -438,7 +438,7 @@ class BasicRecipe(QWidget):  # {{{
             self.max_articles.setValue(recipe.max_articles_per_feed)
             for x in (recipe.feeds or ()):
                 title, url = ('', x) if len(x) == 1 else x
-                QListWidgetItem('%s - %s' % (title, url), self.feeds).setData(Qt.UserRole, (title, url))
+                QListWidgetItem('%s - %s' % (title, url), self.feeds).setData(Qt.ItemDataRole.UserRole, (title, url))
 
 # }}}
 
@@ -484,7 +484,7 @@ class ChooseBuiltinRecipeModel(QSortFilterProxyModel):
 
     def filterAcceptsRow(self, source_row, source_parent):
         idx = self.sourceModel().index(source_row, 0, source_parent)
-        urn = idx.data(Qt.UserRole)
+        urn = idx.data(Qt.ItemDataRole.UserRole)
         if not urn or urn in ('::category::0', '::category::1'):
             return False
         return True
@@ -509,7 +509,7 @@ class ChooseBuiltinRecipe(Dialog):  # {{{
         self.search.initialize('scheduler_search_history')
         self.search.setMinimumContentsLength(15)
         self.search.search.connect(self.recipe_model.search)
-        self.recipe_model.searched.connect(self.search.search_done, type=Qt.QueuedConnection)
+        self.recipe_model.searched.connect(self.search.search_done, type=Qt.ConnectionType.QueuedConnection)
         self.recipe_model.searched.connect(self.search_done)
         self.go_button = b = QToolButton(self)
         b.setText(_("Go"))
@@ -519,7 +519,7 @@ class ChooseBuiltinRecipe(Dialog):  # {{{
         l.addLayout(h)
         l.addWidget(self.recipes)
         l.addWidget(self.bb)
-        self.search.setFocus(Qt.OtherFocusReason)
+        self.search.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def search_done(self, *args):
         if self.recipe_model.showing_count < 10:
@@ -531,7 +531,7 @@ class ChooseBuiltinRecipe(Dialog):  # {{{
     @property
     def selected_recipe(self):
         for idx in self.recipes.selectedIndexes():
-            urn = idx.data(Qt.UserRole)
+            urn = idx.data(Qt.ItemDataRole.UserRole)
             if urn and not urn.startswith('::category::'):
                 return urn
 
@@ -589,7 +589,7 @@ class CustomRecipes(Dialog):
                 b.setIcon(QIcon(I(icon))), b.setToolTip(tooltip)
                 b.clicked.connect(receiver)
         else:
-            bb.setStandardButtons(bb.Cancel | bb.Save)
+            bb.setStandardButtons(QDialogButtonBox.StandardButton.Cancel | QDialogButtonBox.StandardButton.Save)
             if self.stack.currentIndex() == 1:
                 text = _('S&witch to advanced mode')
                 tooltip = _('Edit this recipe in advanced mode')
@@ -656,7 +656,7 @@ class CustomRecipes(Dialog):
 
     def customize_recipe(self):
         d = ChooseBuiltinRecipe(self.recipe_model, self)
-        if d.exec_() != d.Accepted:
+        if d.exec_() != QDialog.DialogCode.Accepted:
             return
 
         id_ = d.selected_recipe
@@ -688,7 +688,7 @@ class CustomRecipes(Dialog):
     def import_opml(self):
         from calibre.gui2.dialogs.opml import ImportOPML
         d = ImportOPML(parent=self)
-        if d.exec_() != d.Accepted:
+        if d.exec_() != QDialog.DialogCode.Accepted:
             return
         oldest_article, max_articles_per_feed, replace_existing = d.oldest_article, d.articles_per_feed, d.replace_existing
         failed_recipes, replace_recipes, add_recipes = {}, {}, {}
