@@ -1,4 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
+
+
 __copyright__ = '2008, Kovid Goyal kovid@kovidgoyal.net'
 __docformat__ = 'restructuredtext en'
 __license__   = 'GPL v3'
@@ -9,7 +11,7 @@ from PyQt5.Qt import (Qt, QDialog, QDialogButtonBox, QSyntaxHighlighter, QFont,
                       QRegExp, QApplication, QTextCharFormat, QColor, QCursor,
                       QIcon, QSize)
 
-from calibre import sanitize_file_name_unicode
+from calibre import sanitize_file_name
 from calibre.constants import config_dir
 from calibre.gui2 import gprefs
 from calibre.gui2.dialogs.template_dialog_ui import Ui_TemplateDialog
@@ -20,6 +22,8 @@ from calibre.ebooks.metadata.book.formatter import SafeFormat
 from calibre.library.coloring import (displayable_columns, color_row_key)
 from calibre.gui2 import error_dialog, choose_files, pixmap_to_data
 from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import native_string_type, unicode_type
+
 
 class ParenPosition:
 
@@ -32,6 +36,7 @@ class ParenPosition:
     def set_highlight(self, to_what):
         self.highlight = to_what
 
+
 class TemplateHighlighter(QSyntaxHighlighter):
 
     Config = {}
@@ -39,7 +44,7 @@ class TemplateHighlighter(QSyntaxHighlighter):
     Formats = {}
     BN_FACTOR = 1000
 
-    KEYWORDS = ["program"]
+    KEYWORDS = ["program", 'if', 'then', 'else', 'elif', 'fi']
 
     def __init__(self, parent=None):
         super(TemplateHighlighter, self).__init__(parent)
@@ -77,15 +82,16 @@ class TemplateHighlighter(QSyntaxHighlighter):
     def initializeFormats(self):
         Config = self.Config
         Config["fontfamily"] = "monospace"
+        pal = QApplication.instance().palette()
         for name, color, bold, italic in (
-                ("normal", "#000000", False, False),
-                ("keyword", "#000080", True, False),
-                ("builtin", "#0000A0", False, False),
+                ("normal", None, False, False),
+                ("keyword", pal.color(pal.Link).name(), True, False),
+                ("builtin", pal.color(pal.Link).name(), False, False),
                 ("comment", "#007F00", False, True),
                 ("string", "#808000", False, False),
                 ("number", "#924900", False, False),
-                ("lparen", "#000000", True, True),
-                ("rparen", "#000000", True, True)):
+                ("lparen", None, True, True),
+                ("rparen", None, True, True)):
             Config["%sfontcolor" % name] = color
             Config["%sfontbold" % name] = bold
             Config["%sfontitalic" % name] = italic
@@ -97,9 +103,11 @@ class TemplateHighlighter(QSyntaxHighlighter):
         for name in ("normal", "keyword", "builtin", "comment",
                      "string", "number", "lparen", "rparen"):
             format = QTextCharFormat(baseFormat)
-            format.setForeground(QColor(Config["%sfontcolor" % name]))
+            col = Config["%sfontcolor" % name]
+            if col:
+                format.setForeground(QColor(col))
             if Config["%sfontbold" % name]:
-                format.setFontWeight(QFont.Bold)
+                format.setFontWeight(QFont.Weight.Bold)
             format.setFontItalic(Config["%sfontitalic" % name])
             self.Formats[name] = format
 
@@ -132,7 +140,7 @@ class TemplateHighlighter(QSyntaxHighlighter):
                 i = regex.indexIn(text, i + length)
 
         if self.generate_paren_positions:
-            t = unicode(text)
+            t = unicode_type(text)
             i = 0
             foundQuote = False
             while i < len(t):
@@ -158,7 +166,7 @@ class TemplateHighlighter(QSyntaxHighlighter):
                 i += 1
 
     def rehighlight(self):
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
+        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
         QSyntaxHighlighter.rehighlight(self)
         QApplication.restoreOverrideCursor()
 
@@ -200,11 +208,12 @@ class TemplateHighlighter(QSyntaxHighlighter):
         self.rehighlight()
         self.generate_paren_positions = False
 
+
 class TemplateDialog(QDialog, Ui_TemplateDialog):
 
     def __init__(self, parent, text, mi=None, fm=None, color_field=None,
                  icon_field_key=None, icon_rule_kind=None, doing_emblem=False,
-                 text_is_placeholder=False):
+                 text_is_placeholder=False, dialog_is_st_editor=False):
         QDialog.__init__(self, parent)
         Ui_TemplateDialog.__init__(self)
         self.setupUi(self)
@@ -212,11 +221,12 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
         self.coloring = color_field is not None
         self.iconing = icon_field_key is not None
         self.embleming = doing_emblem
+        self.dialog_is_st_editor = dialog_is_st_editor
 
         cols = []
         if fm is not None:
             for key in sorted(displayable_columns(fm),
-                              key=lambda(k): sort_key(fm[k]['name']) if k != color_row_key else 0):
+                              key=lambda k: sort_key(fm[k]['name'] if k != color_row_key else 0)):
                 if key == color_row_key and not self.coloring:
                     continue
                 from calibre.gui2.preferences.coloring import all_columns_string
@@ -264,6 +274,14 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
                 self.icon_kind.setCurrentIndex(dex)
                 self.icon_field.setCurrentIndex(self.icon_field.findData(icon_field_key))
 
+        if dialog_is_st_editor:
+            self.buttonBox.setVisible(False)
+        else:
+            self.new_doc_label.setVisible(False)
+            self.new_doc.setVisible(False)
+            self.template_name_label.setVisible(False)
+            self.template_name.setVisible(False)
+
         if mi:
             self.mi = mi
         else:
@@ -285,10 +303,12 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
                 from calibre.gui2.ui import get_gui
                 self.mi.set_all_user_metadata(
                       get_gui().current_db.new_api.field_metadata.custom_field_metadata())
+            for col in self.mi.get_all_user_metadata(False):
+                self.mi.set(col, (col,), 0)
 
         # Remove help icon on title bar
         icon = self.windowIcon()
-        self.setWindowFlags(self.windowFlags()&(~Qt.WindowContextHelpButtonHint))
+        self.setWindowFlags(self.windowFlags()&(~Qt.WindowType.WindowContextHelpButtonHint))
         self.setWindowIcon(icon)
 
         self.last_text = ''
@@ -304,10 +324,11 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
         if text is not None:
             if text_is_placeholder:
                 self.textbox.setPlaceholderText(text)
+                self.textbox.clear()
             else:
                 self.textbox.setPlainText(text)
-        self.buttonBox.button(QDialogButtonBox.Ok).setText(_('&OK'))
-        self.buttonBox.button(QDialogButtonBox.Cancel).setText(_('&Cancel'))
+        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setText(_('&OK'))
+        self.buttonBox.button(QDialogButtonBox.StandardButton.Cancel).setText(_('&Cancel'))
         self.color_copy_button.clicked.connect(self.color_to_clipboard)
         self.filename_button.clicked.connect(self.filename_button_clicked)
         self.icon_copy_button.clicked.connect(self.icon_to_clipboard)
@@ -326,7 +347,7 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
         self.function.addItem('')
         self.function.addItems(func_names)
         self.function.setCurrentIndex(0)
-        self.function.currentIndexChanged[str].connect(self.function_changed)
+        self.function.currentIndexChanged[native_string_type].connect(self.function_changed)
         self.textbox_changed()
         self.rule = (None, '')
 
@@ -337,7 +358,7 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
         tt = _('Template function reference')
         self.template_func_reference.setText(
             '<a href="%s">%s</a>' % (
-                localize_user_manual_link('https://manual.calibre-ebook.com/template_ref.html'), tt))
+                localize_user_manual_link('https://manual.calibre-ebook.com/generated/en/template_ref.html'), tt))
 
         self.font_size_box.setValue(gprefs['gpm_template_editor_font_size'])
         self.font_size_box.valueChanged.connect(self.font_size_changed)
@@ -350,12 +371,12 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
     def filename_button_clicked(self):
         try:
             path = choose_files(self, 'choose_category_icon',
-                        _('Select Icon'), filters=[
+                        _('Select icon'), filters=[
                         ('Images', ['png', 'gif', 'jpg', 'jpeg'])],
                     all_files=False, select_only_single_file=True)
             if path:
                 icon_path = path[0]
-                icon_name = sanitize_file_name_unicode(
+                icon_name = sanitize_file_name(
                              os.path.splitext(
                                    os.path.basename(icon_path))[0]+'.png')
                 if icon_name not in self.icon_file_names:
@@ -389,15 +410,15 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
     def color_to_clipboard(self):
         app = QApplication.instance()
         c = app.clipboard()
-        c.setText(unicode(self.color_name.color))
+        c.setText(unicode_type(self.color_name.color))
 
     def icon_to_clipboard(self):
         app = QApplication.instance()
         c = app.clipboard()
-        c.setText(unicode(self.icon_files.currentText()))
+        c.setText(unicode_type(self.icon_files.currentText()))
 
     def textbox_changed(self):
-        cur_text = unicode(self.textbox.toPlainText())
+        cur_text = unicode_type(self.textbox.toPlainText())
         if self.last_text != cur_text:
             self.last_text = cur_text
             self.highlighter.regenerate_paren_positions()
@@ -409,7 +430,7 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
     def text_cursor_changed(self):
         cursor = self.textbox.textCursor()
         position = cursor.position()
-        t = unicode(self.textbox.toPlainText())
+        t = unicode_type(self.textbox.toPlainText())
         if position > 0 and position <= len(t):
             block_number = cursor.blockNumber()
             pos_in_block = cursor.positionInBlock() - 1
@@ -417,18 +438,23 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
                                               pos_in_block)
 
     def function_changed(self, toWhat):
-        name = unicode(toWhat)
+        name = unicode_type(toWhat)
         self.source_code.clear()
         self.documentation.clear()
+        self.func_type.clear()
         if name in self.funcs:
             self.documentation.setPlainText(self.funcs[name].doc)
             if name in self.builtins and name in self.builtin_source_dict:
                 self.source_code.setPlainText(self.builtin_source_dict[name])
             else:
                 self.source_code.setPlainText(self.funcs[name].program_text)
+            if self.funcs[name].is_python:
+                self.func_type.setText(_('Template function in Python'))
+            else:
+                self.func_type.setText(_('Stored template'))
 
     def accept(self):
-        txt = unicode(self.textbox.toPlainText()).rstrip()
+        txt = unicode_type(self.textbox.toPlainText()).rstrip()
         if self.coloring:
             if self.colored_field.currentIndex() == -1:
                 error_dialog(self, _('No column chosen'),
@@ -439,12 +465,12 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
                     _('The template box cannot be empty'), show=True)
                 return
 
-            self.rule = (unicode(self.colored_field.itemData(
+            self.rule = (unicode_type(self.colored_field.itemData(
                                 self.colored_field.currentIndex()) or ''), txt)
         elif self.iconing:
-            rt = unicode(self.icon_kind.itemData(self.icon_kind.currentIndex()) or '')
+            rt = unicode_type(self.icon_kind.itemData(self.icon_kind.currentIndex()) or '')
             self.rule = (rt,
-                         unicode(self.icon_field.itemData(
+                         unicode_type(self.icon_field.itemData(
                                 self.icon_field.currentIndex()) or ''),
                          txt)
         elif self.embleming:
@@ -453,6 +479,28 @@ class TemplateDialog(QDialog, Ui_TemplateDialog):
             self.rule = ('', txt)
         QDialog.accept(self)
 
+    def reject(self):
+        QDialog.reject(self)
+        if self.dialog_is_st_editor:
+            parent = self.parent()
+            while True:
+                if hasattr(parent, 'reject'):
+                    parent.reject()
+                    break
+                parent = parent.parent()
+                if parent is None:
+                    break
+
+
+class EmbeddedTemplateDialog(TemplateDialog):
+
+    def __init__(self, parent):
+        TemplateDialog.__init__(self, parent, _('A General Program Mode Template'), text_is_placeholder=True,
+                                dialog_is_st_editor=True)
+        self.setParent(parent)
+        self.setWindowFlags(Qt.WindowType.Widget)
+
+
 if __name__ == '__main__':
     app = QApplication([])
     from calibre.ebooks.metadata.book.base import field_metadata
