@@ -1,29 +1,34 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
+
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 import functools
+from PyQt5.Qt import (
+    QAction, QApplication, QIcon, QLabel, QMenu, QPainter, QSizePolicy, QSplitter,
+    QStackedWidget, QStatusBar, QStyle, QStyleOption, Qt, QTabBar, QTimer,
+    QToolButton, QVBoxLayout, QWidget, QDialog
+)
 
-from PyQt5.Qt import (Qt, QApplication, QStackedWidget, QMenu, QTimer,
-        QSizePolicy, QStatusBar, QLabel, QFont, QAction, QTabBar, QStyle,
-        QVBoxLayout, QWidget, QSplitter, QToolButton, QIcon, QPainter, QStyleOption)
-
+from calibre.constants import __appname__, get_version, ismacos
+from calibre.customize.ui import find_plugin
+from calibre.gui2 import (
+    config, error_dialog, gprefs, is_widescreen, open_local_file, open_url
+)
+from calibre.gui2.book_details import BookDetails
+from calibre.gui2.layout_menu import LayoutMenu
+from calibre.gui2.library.alternate_views import GridView
+from calibre.gui2.library.views import BooksView, DeviceBooksView
+from calibre.gui2.notify import get_notifier
+from calibre.gui2.tag_browser.ui import TagBrowserWidget
+from calibre.gui2.widgets import LayoutButton, Splitter
 from calibre.utils.config import prefs
 from calibre.utils.icu import sort_key
-from calibre.constants import (isosx, __appname__, preferred_encoding,
-    get_version)
-from calibre.gui2 import config, is_widescreen, gprefs, error_dialog, open_url
-from calibre.gui2.library.views import BooksView, DeviceBooksView
-from calibre.gui2.library.alternate_views import GridView
-from calibre.gui2.widgets import Splitter, LayoutButton
-from calibre.gui2.tag_browser.ui import TagBrowserWidget
-from calibre.gui2.book_details import BookDetails
-from calibre.gui2.notify import get_notifier
-from calibre.gui2.layout_menu import LayoutMenu
-from calibre.customize.ui import find_plugin
+from calibre.utils.localization import localize_website_link
+from polyglot.builtins import unicode_type
 
 _keep_refs = []
 
@@ -40,11 +45,11 @@ class LibraryViewMixin(object):  # {{{
         pass
 
     def init_library_view_mixin(self, db):
-        self.library_view.files_dropped.connect(self.iactions['Add Books'].files_dropped, type=Qt.QueuedConnection)
-        self.library_view.books_dropped.connect(self.iactions['Edit Metadata'].books_dropped, type=Qt.QueuedConnection)
+        self.library_view.files_dropped.connect(self.iactions['Add Books'].files_dropped, type=Qt.ConnectionType.QueuedConnection)
+        self.library_view.books_dropped.connect(self.iactions['Edit Metadata'].books_dropped, type=Qt.ConnectionType.QueuedConnection)
         self.library_view.add_column_signal.connect(partial(self.iactions['Preferences'].do_config,
             initial_plugin=('Interface', 'Custom Columns')),
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         for func, args in [
                              ('connect_to_search_box', (self.search,
                                  self.search_done)),
@@ -111,7 +116,7 @@ class LibraryViewMixin(object):  # {{{
 
 class QuickviewSplitter(QSplitter):  # {{{
 
-    def __init__(self, parent=None, orientation=Qt.Vertical, qv_widget=None):
+    def __init__(self, parent=None, orientation=Qt.Orientation.Vertical, qv_widget=None):
         QSplitter.__init__(self, parent=parent, orientation=orientation)
         self.splitterMoved.connect(self.splitter_moved)
         self.setChildrenCollapsible(False)
@@ -144,11 +149,11 @@ class QuickviewSplitter(QSplitter):  # {{{
 class LibraryWidget(Splitter):  # {{{
 
     def __init__(self, parent):
-        orientation = Qt.Vertical
+        orientation = Qt.Orientation.Vertical
         if config['gui_layout'] == 'narrow':
-            orientation = Qt.Horizontal if is_widescreen() else Qt.Vertical
-        idx = 0 if orientation == Qt.Vertical else 1
-        size = 300 if orientation == Qt.Vertical else 550
+            orientation = Qt.Orientation.Horizontal if is_widescreen() else Qt.Orientation.Vertical
+        idx = 0 if orientation == Qt.Orientation.Vertical else 1
+        size = 300 if orientation == Qt.Orientation.Vertical else 550
         Splitter.__init__(self, 'cover_browser_splitter', _('Cover browser'),
                 I('cover_flow.png'),
                 orientation=orientation, parent=parent,
@@ -158,7 +163,7 @@ class LibraryWidget(Splitter):  # {{{
 
         quickview_widget = QWidget()
         parent.quickview_splitter = QuickviewSplitter(
-                parent=self, orientation=Qt.Vertical, qv_widget=quickview_widget)
+                parent=self, orientation=Qt.Orientation.Vertical, qv_widget=quickview_widget)
         parent.library_view = BooksView(parent)
         parent.library_view.setObjectName('library_view')
         stack = QStackedWidget(self)
@@ -191,7 +196,7 @@ class Stack(QStackedWidget):  # {{{
                 parent=parent, side_index=0, initial_side_size=200,
                 shortcut='Shift+Alt+T')
         parent.tb_splitter.state_changed.connect(
-                        self.tb_widget.set_pane_is_visible, Qt.QueuedConnection)
+                        self.tb_widget.set_pane_is_visible, Qt.ConnectionType.QueuedConnection)
         parent.tb_splitter.addWidget(self.tb_widget)
         parent.tb_splitter.addWidget(parent.cb_splitter)
         parent.tb_splitter.setCollapsible(parent.tb_splitter.other_index, False)
@@ -211,7 +216,7 @@ class UpdateLabel(QLabel):  # {{{
 
     def __init__(self, *args, **kwargs):
         QLabel.__init__(self, *args, **kwargs)
-        self.setCursor(Qt.PointingHandCursor)
+        self.setCursor(Qt.CursorShape.PointingHandCursor)
 
     def contextMenuEvent(self, e):
         pass
@@ -223,11 +228,11 @@ class VersionLabel(QLabel):  # {{{
     def __init__(self, parent):
         QLabel.__init__(self, parent)
         self.mouse_over = False
-        self.setCursor(Qt.PointingHandCursor)
+        self.setCursor(Qt.CursorShape.PointingHandCursor)
         self.setToolTip(_('See what\'s new in this calibre release'))
 
     def mouseReleaseEvent(self, ev):
-        open_url('https://calibre-ebook.com/whats-new')
+        open_url(localize_website_link('https://calibre-ebook.com/whats-new'))
         ev.accept()
         return QLabel.mouseReleaseEvent(self, ev)
 
@@ -248,9 +253,9 @@ class VersionLabel(QLabel):  # {{{
             p = QPainter(self)
             tool = QStyleOption()
             tool.rect = self.rect()
-            tool.state = QStyle.State_Raised | QStyle.State_Active | QStyle.State_MouseOver
+            tool.state = QStyle.StateFlag.State_Raised | QStyle.StateFlag.State_Active | QStyle.StateFlag.State_MouseOver
             s = self.style()
-            s.drawPrimitive(QStyle.PE_PanelButtonTool, tool, p, self)
+            s.drawPrimitive(QStyle.PrimitiveElement.PE_PanelButtonTool, tool, p, self)
             p.end()
         return QLabel.paintEvent(self, ev)
 # }}}
@@ -267,11 +272,7 @@ class StatusBar(QStatusBar):  # {{{
         self.total = self.current = self.selected = self.library_total = 0
         self.addPermanentWidget(self.update_label)
         self.update_label.setVisible(False)
-        self._font = QFont()
-        self._font.setBold(True)
-        self.setFont(self._font)
         self.defmsg = VersionLabel(self)
-        self.defmsg.setFont(self._font)
         self.addWidget(self.defmsg)
         self.set_label()
 
@@ -311,7 +312,7 @@ class StatusBar(QStatusBar):  # {{{
         if self.library_total != self.total:
             base = _('{0}, {1} total').format(base, self.library_total)
 
-        self.defmsg.setText(u'\xa0%s\xa0\xa0\xa0\xa0[%s]' % (msg, base))
+        self.defmsg.setText('\xa0%s\xa0\xa0\xa0\xa0[%s]' % (msg, base))
         self.clearMessage()
 
     def device_disconnected(self):
@@ -321,11 +322,6 @@ class StatusBar(QStatusBar):  # {{{
     def show_message(self, msg, timeout=0, show_notification=True):
         self.showMessage(msg, timeout)
         if self.notifier is not None and not config['disable_tray_notification'] and show_notification:
-            if isosx and isinstance(msg, unicode):
-                try:
-                    msg = msg.encode(preferred_encoding)
-                except UnicodeEncodeError:
-                    msg = msg.encode('utf-8')
             self.notifier(msg)
 
     def clear_message(self):
@@ -342,7 +338,7 @@ class GridViewButton(LayoutButton):  # {{{
         self.set_state_to_show()
         self.action_toggle = QAction(self.icon(), _('Toggle') + ' ' + self.label, self)
         gui.addAction(self.action_toggle)
-        gui.keyboard.register_shortcut('grid view toggle' + self.label, unicode(self.action_toggle.text()),
+        gui.keyboard.register_shortcut('grid view toggle' + self.label, unicode_type(self.action_toggle.text()),
                                     default_keys=(sc,), action=self.action_toggle)
         self.action_toggle.triggered.connect(self.toggle)
         self.action_toggle.changed.connect(self.update_shortcut)
@@ -372,7 +368,7 @@ class SearchBarButton(LayoutButton):  # {{{
         self.set_state_to_hide()
         self.action_toggle = QAction(self.icon(), _('Toggle') + ' ' + self.label, self)
         gui.addAction(self.action_toggle)
-        gui.keyboard.register_shortcut('search bar toggle' + self.label, unicode(self.action_toggle.text()),
+        gui.keyboard.register_shortcut('search bar toggle' + self.label, unicode_type(self.action_toggle.text()),
                                     default_keys=(sc,), action=self.action_toggle)
         self.action_toggle.triggered.connect(self.toggle)
         self.action_toggle.changed.connect(self.update_shortcut)
@@ -404,21 +400,20 @@ class VLTabs(QTabBar):  # {{{
         self.gui = parent
         self.ignore_tab_changed = False
         self.currentChanged.connect(self.tab_changed)
-        self.tabMoved.connect(self.tab_moved, type=Qt.QueuedConnection)
+        self.tabMoved.connect(self.tab_moved, type=Qt.ConnectionType.QueuedConnection)
         self.tabCloseRequested.connect(self.tab_close)
-        self.setStyleSheet('QTabBar::tab:selected { font-weight: bold } QTabBar::tab { text-align: center }')
         self.setVisible(gprefs['show_vl_tabs'])
         self.next_action = a = QAction(self)
         a.triggered.connect(partial(self.next_tab, delta=1)), self.gui.addAction(a)
         self.previous_action = a = QAction(self)
         a.triggered.connect(partial(self.next_tab, delta=-1)), self.gui.addAction(a)
         self.gui.keyboard.register_shortcut(
-            'virtual-library-tab-bar-next', _('Next virtual library'), action=self.next_action,
+            'virtual-library-tab-bar-next', _('Next Virtual library'), action=self.next_action,
             default_keys=('Ctrl+Right',),
             description=_('Switch to the next Virtual library in the Virtual library tab bar')
         )
         self.gui.keyboard.register_shortcut(
-            'virtual-library-tab-bar-previous', _('Previous virtual library'), action=self.previous_action,
+            'virtual-library-tab-bar-previous', _('Previous Virtual library'), action=self.previous_action,
             default_keys=('Ctrl+Left',),
             description=_('Switch to the previous Virtual library in the Virtual library tab bar')
         )
@@ -431,10 +426,12 @@ class VLTabs(QTabBar):  # {{{
     def enable_bar(self):
         gprefs['show_vl_tabs'] = True
         self.setVisible(True)
+        self.gui.set_number_of_books_shown()
 
     def disable_bar(self):
         gprefs['show_vl_tabs'] = False
         self.setVisible(False)
+        self.gui.set_number_of_books_shown()
 
     def lock_tab(self):
         gprefs['vl_tabs_closable'] = False
@@ -456,17 +453,17 @@ class VLTabs(QTabBar):  # {{{
     def tab_changed(self, idx):
         if self.ignore_tab_changed:
             return
-        vl = unicode(self.tabData(idx) or '').strip() or None
+        vl = unicode_type(self.tabData(idx) or '').strip() or None
         self.gui.apply_virtual_library(vl, update_tabs=False)
 
     def tab_moved(self, from_, to):
-        self.current_db.new_api.set_pref('virt_libs_order', [unicode(self.tabData(i) or '') for i in range(self.count())])
+        self.current_db.new_api.set_pref('virt_libs_order', [unicode_type(self.tabData(i) or '') for i in range(self.count())])
 
     def tab_close(self, index):
-        vl = unicode(self.tabData(index) or '')
+        vl = unicode_type(self.tabData(index) or '')
         if vl:  # Dont allow closing the All Books tab
             self.current_db.new_api.set_pref('virt_libs_hidden', list(
-                self.current_db.prefs['virt_libs_hidden']) + [vl])
+                self.current_db.new_api.pref('virt_libs_hidden', ())) + [vl])
             self.removeTab(index)
 
     @property
@@ -482,13 +479,13 @@ class VLTabs(QTabBar):  # {{{
 
     def _rebuild(self):
         db = self.current_db
-        vl_map = db.prefs.get('virtual_libraries', {})
+        vl_map = db.new_api.pref('virtual_libraries', {})
         virt_libs = frozenset(vl_map)
-        hidden = set(db.prefs['virt_libs_hidden'])
+        hidden = set(db.new_api.pref('virt_libs_hidden', ()))
         if hidden - virt_libs:
             hidden = hidden.intersection(virt_libs)
             db.new_api.set_pref('virt_libs_hidden', list(hidden))
-        order = db.prefs['virt_libs_order']
+        order = db.new_api.pref('virt_libs_order', ())
         while self.count():
             self.removeTab(0)
         current_lib = db.data.get_base_restriction_name()
@@ -502,7 +499,7 @@ class VLTabs(QTabBar):  # {{{
             self.addTab(vl.replace('&', '&&') or _('All books'))
             sexp = vl_map.get(vl, None)
             if sexp is not None:
-                self.setTabToolTip(i, _('Search expression for this virtual library:') + '\n\n' + sexp)
+                self.setTabToolTip(i, _('Search expression for this Virtual library:') + '\n\n' + sexp)
             self.setTabData(i, vl)
             if vl == current_lib:
                 current_idx = i
@@ -527,19 +524,19 @@ class VLTabs(QTabBar):  # {{{
     def contextMenuEvent(self, ev):
         m = QMenu(self)
         m.addAction(_('Sort tabs alphabetically'), self.sort_alphabetically)
-        hidden = self.current_db.prefs['virt_libs_hidden']
+        hidden = self.current_db.new_api.pref('virt_libs_hidden')
         if hidden:
             s = m._s = m.addMenu(_('Restore hidden tabs'))
             for x in hidden:
                 s.addAction(x, partial(self.restore, x))
-        m.addAction(_('Hide virtual library tabs'), self.disable_bar)
+        m.addAction(_('Hide Virtual library tabs'), self.disable_bar)
         if gprefs['vl_tabs_closable']:
-            m.addAction(_('Lock virtual library tabs'), self.lock_tab)
+            m.addAction(_('Lock Virtual library tabs'), self.lock_tab)
         else:
-            m.addAction(_('Unlock virtual library tabs'), self.unlock_tab)
+            m.addAction(_('Unlock Virtual library tabs'), self.unlock_tab)
         i = self.tabAt(ev.pos())
         if i > -1:
-            vl = unicode(self.tabData(i) or '')
+            vl = unicode_type(self.tabData(i) or '')
             if vl:
                 m.addSeparator()
                 m.addAction(_('Edit "%s"') % vl, partial(self.gui.do_create_edit, name=vl))
@@ -551,7 +548,7 @@ class VLTabs(QTabBar):  # {{{
         self.rebuild()
 
     def restore(self, x):
-        h = self.current_db.prefs['virt_libs_hidden']
+        h = self.current_db.new_api.pref('virt_libs_hidden', ())
         self.current_db.new_api.set_pref('virt_libs_hidden', list(set(h) - {x}))
         self.rebuild()
 
@@ -572,7 +569,7 @@ class LayoutMixin(object):  # {{{
             self.stack = Stack(self)
             self.bd_splitter = Splitter('book_details_splitter',
                     _('Book details'), I('book.png'),
-                    orientation=Qt.Vertical, parent=self, side_index=1,
+                    orientation=Qt.Orientation.Vertical, parent=self, side_index=1,
                     shortcut='Shift+Alt+D')
             self.bd_splitter.addWidget(self.stack)
             self.bd_splitter.addWidget(self.book_details)
@@ -583,15 +580,15 @@ class LayoutMixin(object):  # {{{
         else:  # wide {{{
             self.bd_splitter = Splitter('book_details_splitter',
                     _('Book details'), I('book.png'), initial_side_size=200,
-                    orientation=Qt.Horizontal, parent=self, side_index=1,
+                    orientation=Qt.Orientation.Horizontal, parent=self, side_index=1,
                     shortcut='Shift+Alt+D')
             self.stack = Stack(self)
             self.bd_splitter.addWidget(self.stack)
             self.book_details = BookDetails(True, self)
             self.bd_splitter.addWidget(self.book_details)
             self.bd_splitter.setCollapsible(self.bd_splitter.other_index, False)
-            self.bd_splitter.setSizePolicy(QSizePolicy(QSizePolicy.Expanding,
-                QSizePolicy.Expanding))
+            self.bd_splitter.setSizePolicy(QSizePolicy(QSizePolicy.Policy.Expanding,
+                QSizePolicy.Policy.Expanding))
             self.centralwidget.layout().addWidget(self.bd_splitter)
             button_order = ('sb', 'tb', 'cb', 'gv', 'qv', 'bd')
         # }}}
@@ -603,7 +600,7 @@ class LayoutMixin(object):  # {{{
             self.qv = self.qv.actual_plugin_
 
         self.status_bar = StatusBar(self)
-        stylename = unicode(self.style().objectName())
+        stylename = unicode_type(self.style().objectName())
         self.grid_view_button = GridViewButton(self)
         self.search_bar_button = SearchBarButton(self)
         self.grid_view_button.toggled.connect(self.toggle_grid_view)
@@ -624,7 +621,7 @@ class LayoutMixin(object):  # {{{
                     button = self.search_bar_button
             self.layout_buttons.append(button)
             button.setVisible(False)
-            if isosx and stylename != u'Calibre':
+            if ismacos and stylename != 'Calibre':
                 button.setStyleSheet('''
                         QToolButton { background: none; border:none; padding: 0px; }
                         QToolButton:checked { background: rgba(0, 0, 0, 25%); }
@@ -636,9 +633,9 @@ class LayoutMixin(object):  # {{{
                 self.status_bar.addPermanentWidget(b)
         else:
             self.layout_button = b = QToolButton(self)
-            b.setAutoRaise(True), b.setCursor(Qt.PointingHandCursor)
-            b.setPopupMode(b.InstantPopup)
-            b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+            b.setAutoRaise(True), b.setCursor(Qt.CursorShape.PointingHandCursor)
+            b.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
+            b.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
             b.setText(_('Layout')), b.setIcon(QIcon(I('config.png')))
             b.setMenu(LayoutMenu(self))
             b.setToolTip(_(
@@ -653,16 +650,18 @@ class LayoutMixin(object):  # {{{
         self.book_details.show_book_info.connect(self.iactions['Show Book Details'].show_book_info)
         self.book_details.files_dropped.connect(self.iactions['Add Books'].files_dropped_on_book)
         self.book_details.cover_changed.connect(self.bd_cover_changed,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.book_details.open_cover_with.connect(self.bd_open_cover_with,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.book_details.open_fmt_with.connect(self.bd_open_fmt_with,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
+        self.book_details.edit_book.connect(self.bd_edit_book,
+                type=Qt.ConnectionType.QueuedConnection)
         self.book_details.cover_removed.connect(self.bd_cover_removed,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.book_details.remote_file_dropped.connect(
                 self.iactions['Add Books'].remote_file_dropped_on_book,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.book_details.open_containing_folder.connect(self.iactions['View'].view_folder_for_id)
         self.book_details.view_specific_format.connect(self.iactions['View'].view_format_by_id)
         self.book_details.search_requested.connect(self.search.set_search_string)
@@ -677,10 +676,12 @@ class LayoutMixin(object):  # {{{
         self.book_details.set_cover_from_format.connect(
             self.iactions['Edit Metadata'].set_cover_from_format)
         self.book_details.copy_link.connect(self.bd_copy_link,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self.book_details.view_device_book.connect(
                 self.iactions['View'].view_device_book)
         self.book_details.manage_category.connect(self.manage_category_triggerred)
+        self.book_details.find_in_tag_browser.connect(self.find_in_tag_browser_triggered)
+        self.book_details.edit_identifiers.connect(self.edit_identifiers_triggerred)
         self.book_details.compare_specific_format.connect(self.compare_format)
 
         m = self.library_view.model()
@@ -688,15 +689,34 @@ class LayoutMixin(object):  # {{{
             QTimer.singleShot(0, self.library_view.set_current_row)
             m.current_changed(self.library_view.currentIndex(),
                     self.library_view.currentIndex())
-        self.library_view.setFocus(Qt.OtherFocusReason)
+        self.library_view.setFocus(Qt.FocusReason.OtherFocusReason)
+
+    def edit_identifiers_triggerred(self):
+        book_id = self.library_view.current_book
+        db = self.current_db.new_api
+        identifiers = db.field_for('identifiers', book_id, default_value={})
+        from calibre.gui2.metadata.basic_widgets import Identifiers
+        d = Identifiers(identifiers, self)
+        if d.exec_() == QDialog.DialogCode.Accepted:
+            identifiers = d.get_identifiers()
+            db.set_field('identifiers', {book_id: identifiers})
+            self.iactions['Edit Metadata'].refresh_books_after_metadata_edit({book_id})
 
     def manage_category_triggerred(self, field, value):
         if field and value:
             if field == 'authors':
-                self.do_author_sort_edit(self, value, select_sort=False, select_link=False)
+                self.do_author_sort_edit(self, value, select_sort=False,
+                                         select_link=False, lookup_author=True)
             elif field:
                 self.do_tags_list_edit(value, field)
 
+    def find_in_tag_browser_triggered(self, field, value):
+        if field and value:
+            tb = self.stack.tb_widget
+            tb.set_focus_to_find_box()
+            tb.item_search.lineEdit().setText(field + ':=' + value)
+            tb.do_find()
+
     def toggle_grid_view(self, show):
         self.library_view.alternate_views.show_view('grid' if show else None)
         self.sort_sep.setVisible(show)
@@ -705,7 +725,7 @@ class LayoutMixin(object):  # {{{
     def toggle_search_bar(self, show):
         self.search_bar.setVisible(show)
         if show:
-            self.search.setFocus(Qt.OtherFocusReason)
+            self.search.setFocus(Qt.FocusReason.OtherFocusReason)
 
     def bd_cover_changed(self, id_, cdata):
         self.library_view.model().db.set_cover(id_, cdata)
@@ -714,6 +734,9 @@ class LayoutMixin(object):  # {{{
     def bd_open_cover_with(self, book_id, entry):
         cpath = self.current_db.new_api.format_abspath(book_id, '__COVER_INTERNAL__')
         if cpath:
+            if entry is None:
+                open_local_file(cpath)
+                return
             from calibre.gui2.open_with import run_program
             run_program(entry, cpath, self)
 
@@ -729,6 +752,11 @@ class LayoutMixin(object):  # {{{
                     self.current_db.new_api.field_for('title', book_id, default_value=_('Unknown')),
                     fmt), show=True)
 
+    def bd_edit_book(self, book_id, fmt):
+        from calibre.gui2.device import BusyCursor
+        with BusyCursor():
+            self.iactions['Tweak ePub'].ebook_edit_format(book_id, fmt)
+
     def open_with_action_triggerred(self, fmt, entry, *args):
         book_id = self.library_view.current_book
         if book_id is not None:
