@@ -7,10 +7,10 @@ import json
 import os
 from functools import partial
 from PyQt5.Qt import (
-    QApplication, QCheckBox, QComboBox, QCursor, QDateTime, QFont, QFormLayout,
+    QApplication, QCheckBox, QComboBox, QCursor, QDateTime, QFont, QFormLayout, QDialog,
     QHBoxLayout, QIcon, QKeySequence, QLabel, QMenu, QPalette, QPlainTextEdit, QSize,
     QSplitter, Qt, QTextBrowser, QTimer, QToolButton, QTreeWidget, QTreeWidgetItem,
-    QVBoxLayout, QWidget, pyqtSignal
+    QVBoxLayout, QWidget, pyqtSignal, QAbstractItemView, QDialogButtonBox
 )
 
 from calibre import prepare_string_for_xml
@@ -24,7 +24,7 @@ from calibre.gui2.widgets2 import Dialog
 # rendering {{{
 def render_highlight_as_text(hl, lines):
     lines.append(hl['highlighted_text'])
-    date = QDateTime.fromString(hl['timestamp'], Qt.ISODate).toLocalTime().toString(Qt.SystemLocaleShortDate)
+    date = QDateTime.fromString(hl['timestamp'], Qt.DateFormat.ISODate).toLocalTime().toString(Qt.DateFormat.SystemLocaleShortDate)
     lines.append(date)
     notes = hl.get('notes')
     if notes:
@@ -37,7 +37,7 @@ def render_highlight_as_text(hl, lines):
 
 def render_bookmark_as_text(b, lines):
     lines.append(b['title'])
-    date = QDateTime.fromString(b['timestamp'], Qt.ISODate).toLocalTime().toString(Qt.SystemLocaleShortDate)
+    date = QDateTime.fromString(b['timestamp'], Qt.DateFormat.ISODate).toLocalTime().toString(Qt.DateFormat.SystemLocaleShortDate)
     lines.append(date)
     lines.append('')
     lines.append('───')
@@ -79,7 +79,7 @@ class AnnotsResultsDelegate(ResultsDelegate):
 
     def result_data(self, result):
         if not isinstance(result, dict):
-            return None, None, None, None
+            return None, None, None, None, None
         full_text = result['text'].replace('\x1f', ' ')
         parts = full_text.split('\x1d', 2)
         before = after = ''
@@ -90,9 +90,7 @@ class AnnotsResultsDelegate(ResultsDelegate):
             before, text = parts
         else:
             text = parts[0]
-        if result.get('annotation', {}).get('notes'):
-            before = '•' + (before or '')
-        return False, before, text, after
+        return False, before, text, after, bool(result.get('annotation', {}).get('notes'))
 
 
 # }}}
@@ -125,7 +123,7 @@ class Export(Dialog):  # {{{
         l.addRow(_('Format to export in:'), ef)
         l.addRow(self.bb)
         self.bb.clear()
-        self.bb.addButton(self.bb.Cancel)
+        self.bb.addButton(QDialogButtonBox.StandardButton.Cancel)
         b = self.bb.addButton(_('Copy to clipboard'), self.bb.ActionRole)
         b.clicked.connect(self.copy_to_clipboard)
         b.setIcon(QIcon(I('edit-copy.png')))
@@ -186,7 +184,7 @@ def current_db():
 class BusyCursor(object):
 
     def __enter__(self):
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
+        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
 
     def __exit__(self, *args):
         QApplication.restoreOverrideCursor()
@@ -204,8 +202,8 @@ class ResultsList(QTreeWidget):
     def __init__(self, parent):
         QTreeWidget.__init__(self, parent)
         self.setHeaderHidden(True)
-        self.setSelectionMode(self.ExtendedSelection)
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         self.customContextMenuRequested.connect(self.show_context_menu)
         self.delegate = AnnotsResultsDelegate(self)
         self.setItemDelegate(self.delegate)
@@ -218,7 +216,10 @@ class ResultsList(QTreeWidget):
 
     def show_context_menu(self, pos):
         item = self.itemAt(pos)
-        result = item.data(0, Qt.UserRole)
+        if item is not None:
+            result = item.data(0, Qt.ItemDataRole.UserRole)
+        else:
+            result = None
         items = self.selectedItems()
         m = QMenu(self)
         if isinstance(result, dict):
@@ -236,17 +237,17 @@ class ResultsList(QTreeWidget):
         m.exec_(self.mapToGlobal(pos))
 
     def edit_notes(self, item):
-        r = item.data(0, Qt.UserRole)
+        r = item.data(0, Qt.ItemDataRole.UserRole)
         if isinstance(r, dict):
             self.edit_annotation.emit(r['id'], r['annotation'])
 
     def show_in_calibre(self, item):
-        r = item.data(0, Qt.UserRole)
+        r = item.data(0, Qt.ItemDataRole.UserRole)
         if isinstance(r, dict):
             self.show_book.emit(r['book_id'], r['format'])
 
     def item_activated(self, item):
-        r = item.data(0, Qt.UserRole)
+        r = item.data(0, Qt.ItemDataRole.UserRole)
         if isinstance(r, dict):
             self.open_annotation.emit(r['book_id'], r['format'], r['annotation'])
 
@@ -264,23 +265,23 @@ class ResultsList(QTreeWidget):
             book_id_map[book_id]['matches'].append(result)
         for book_id, entry in book_id_map.items():
             section = QTreeWidgetItem([entry['title']], 1)
-            section.setFlags(Qt.ItemIsEnabled)
+            section.setFlags(Qt.ItemFlag.ItemIsEnabled)
             section.setFont(0, self.section_font)
             self.addTopLevelItem(section)
             section.setExpanded(True)
             for result in entry['matches']:
                 item = QTreeWidgetItem(section, [' '], 2)
                 self.item_map.append(item)
-                item.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemNeverHasChildren)
-                item.setData(0, Qt.UserRole, result)
-                item.setData(0, Qt.UserRole + 1, self.number_of_results)
+                item.setFlags(Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemNeverHasChildren)
+                item.setData(0, Qt.ItemDataRole.UserRole, result)
+                item.setData(0, Qt.ItemDataRole.UserRole + 1, self.number_of_results)
                 self.number_of_results += 1
         if self.item_map:
             self.setCurrentItem(self.item_map[0])
 
     def current_item_changed(self, current, previous):
         if current is not None:
-            r = current.data(0, Qt.UserRole)
+            r = current.data(0, Qt.ItemDataRole.UserRole)
             if isinstance(r, dict):
                 self.current_result_changed.emit(r)
         else:
@@ -290,7 +291,7 @@ class ResultsList(QTreeWidget):
         item = self.currentItem()
         if item is None:
             return
-        i = int(item.data(0, Qt.UserRole + 1))
+        i = int(item.data(0, Qt.ItemDataRole.UserRole + 1))
         i += -1 if backwards else 1
         i %= self.number_of_results
         self.setCurrentItem(self.item_map[i])
@@ -298,19 +299,19 @@ class ResultsList(QTreeWidget):
     @property
     def selected_annot_ids(self):
         for item in self.selectedItems():
-            yield item.data(0, Qt.UserRole)['id']
+            yield item.data(0, Qt.ItemDataRole.UserRole)['id']
 
     @property
     def selected_annotations(self):
         for item in self.selectedItems():
-            x = item.data(0, Qt.UserRole)
+            x = item.data(0, Qt.ItemDataRole.UserRole)
             ans = x['annotation'].copy()
             for key in ('book_id', 'format'):
                 ans[key] = x[key]
             yield ans
 
     def keyPressEvent(self, ev):
-        if ev.matches(QKeySequence.Delete):
+        if ev.matches(QKeySequence.StandardKey.Delete):
             self.delete_requested.emit()
             ev.accept()
             return
@@ -442,21 +443,21 @@ class BrowsePanel(QWidget):
         l.addLayout(h)
         self.search_box = sb = SearchBox(self)
         sb.initialize('library-annotations-browser-search-box')
-        sb.cleared.connect(self.cleared, type=Qt.QueuedConnection)
+        sb.cleared.connect(self.cleared, type=Qt.ConnectionType.QueuedConnection)
         sb.lineEdit().returnPressed.connect(self.show_next)
         sb.lineEdit().setPlaceholderText(_('Enter words to search for'))
         h.addWidget(sb)
 
         self.next_button = nb = QToolButton(self)
         h.addWidget(nb)
-        nb.setFocusPolicy(Qt.NoFocus)
+        nb.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         nb.setIcon(QIcon(I('arrow-down.png')))
         nb.clicked.connect(self.show_next)
         nb.setToolTip(_('Find next match'))
 
         self.prev_button = nb = QToolButton(self)
         h.addWidget(nb)
-        nb.setFocusPolicy(Qt.NoFocus)
+        nb.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         nb.setIcon(QIcon(I('arrow-up.png')))
         nb.clicked.connect(self.show_previous)
         nb.setToolTip(_('Find previous match'))
@@ -477,7 +478,7 @@ class BrowsePanel(QWidget):
 
     def re_initialize(self, restrict_to_book_ids=None):
         db = current_db()
-        self.search_box.setFocus(Qt.OtherFocusReason)
+        self.search_box.setFocus(Qt.FocusReason.OtherFocusReason)
         self.restrictions.re_initialize(db, restrict_to_book_ids or set())
         self.current_query = None
         self.results_list.clear()
@@ -566,9 +567,9 @@ class Details(QTextBrowser):
         QTextBrowser.__init__(self, parent)
         self.setFrameShape(self.NoFrame)
         self.setOpenLinks(False)
-        self.setAttribute(Qt.WA_OpaquePaintEvent, False)
+        self.setAttribute(Qt.WidgetAttribute.WA_OpaquePaintEvent, False)
         palette = self.palette()
-        palette.setBrush(QPalette.Base, Qt.transparent)
+        palette.setBrush(QPalette.ColorRole.Base, Qt.GlobalColor.transparent)
         self.setPalette(palette)
         self.setAcceptDrops(False)
 
@@ -662,7 +663,7 @@ class DetailsPanel(QWidget):
                     _('Add notes to this highlight'), _('Add notes')))
 
         annot_text += '\n'.join(paras)
-        date = QDateTime.fromString(annot['timestamp'], Qt.ISODate).toLocalTime().toString(Qt.SystemLocaleShortDate)
+        date = QDateTime.fromString(annot['timestamp'], Qt.DateFormat.ISODate).toLocalTime().toString(Qt.DateFormat.SystemLocaleShortDate)
 
         text = '''
         <style>a {{ text-decoration: none }}</style>
@@ -719,7 +720,7 @@ class AnnotationsBrowser(Dialog):
 
     def __init__(self, parent=None):
         Dialog.__init__(self, _('Annotations browser'), 'library-annotations-browser', parent=parent)
-        self.setAttribute(Qt.WA_DeleteOnClose, False)
+        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, False)
         self.setWindowIcon(QIcon(I('highlight.png')))
 
     def do_open_annotation(self, book_id, fmt, annot):
@@ -732,7 +733,7 @@ class AnnotationsBrowser(Dialog):
             self.open_annotation.emit(book_id, fmt, 'epubcfi(/{}{})'.format(x, annot['start_cfi']))
 
     def keyPressEvent(self, ev):
-        if ev.key() not in (Qt.Key_Enter, Qt.Key_Return):
+        if ev.key() not in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
             return Dialog.keyPressEvent(self, ev)
 
     def setup_ui(self):
@@ -741,7 +742,7 @@ class AnnotationsBrowser(Dialog):
         us.setToolTip('<p>' + _(
             'With this option searching for words will also match on any related English words. For'
             ' example: <i>correction</i> matches <i>correcting</i> and <i>corrected</i> as well'))
-        us.stateChanged.connect(lambda state: gprefs.set('browse_annots_use_stemmer', state != Qt.Unchecked))
+        us.stateChanged.connect(lambda state: gprefs.set('browse_annots_use_stemmer', state != Qt.CheckState.Unchecked))
 
         l = QVBoxLayout(self)
 
@@ -810,7 +811,7 @@ class AnnotationsBrowser(Dialog):
                 'Editing is only supported for the notes associated with highlights'), show=True)
         notes = annot.get('notes')
         d = EditNotes(notes, self)
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             notes = d.notes
             if notes and notes.strip():
                 annot['notes'] = notes.strip()
