@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -12,10 +11,11 @@ from collections import namedtuple
 from PyQt5.Qt import (
     QWidget, Qt, QLabel, QVBoxLayout, QDialogButtonBox, QApplication, QTimer, QPixmap,
     QSize, pyqtSignal, QIcon, QPlainTextEdit, QCheckBox, QPainter, QHBoxLayout, QFontMetrics,
-    QPainterPath, QRectF, pyqtProperty, QPropertyAnimation, QEasingCurve, QSizePolicy, QImage)
+    QPainterPath, QRectF, pyqtProperty, QPropertyAnimation, QEasingCurve, QSizePolicy, QImage, QPalette)
 
 from calibre.constants import __version__
 from calibre.gui2.dialogs.message_box import ViewLog
+from polyglot.builtins import unicode_type
 
 Question = namedtuple('Question', 'payload callback cancel_callback '
         'title msg html_log log_viewer_title log_is_file det_msg '
@@ -45,12 +45,12 @@ class Icon(QWidget):
 
     def __init__(self, parent):
         QWidget.__init__(self, parent)
-        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
+        self.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
         self.set_icon('dialog_question.png')
         self.default_icon = self.icon
         self._fraction = 0.0
         self.animation = a = QPropertyAnimation(self, b"fraction", self)
-        a.setDuration(2000), a.setEasingCurve(QEasingCurve.Linear)
+        a.setDuration(2000), a.setEasingCurve(QEasingCurve.Type.Linear)
         a.setStartValue(0.0), a.setEndValue(2.0), a.setLoopCount(10)
 
     def set_icon(self, icon):
@@ -101,7 +101,7 @@ class ProceedQuestion(QWidget):
 
         self._show_fraction = 0.0
         self.show_animation = a = QPropertyAnimation(self, b"show_fraction", self)
-        a.setDuration(1000), a.setEasingCurve(QEasingCurve.OutQuad)
+        a.setDuration(1000), a.setEasingCurve(QEasingCurve.Type.OutQuad)
         a.setStartValue(0.0), a.setEndValue(1.0)
         a.finished.connect(self.stop_show_animation)
         self.rendered_pixmap = None
@@ -130,8 +130,9 @@ class ProceedQuestion(QWidget):
                 _('Show detailed information about this error'))
         self.det_msg = PlainTextEdit(self)
         self.det_msg.setReadOnly(True)
-        self.bb.setStandardButtons(self.bb.Yes|self.bb.No|self.bb.Ok)
-        self.bb.button(self.bb.Yes).setDefault(True)
+        self.bb.setStandardButtons(
+            QDialogButtonBox.StandardButton.Yes | QDialogButtonBox.StandardButton.No | QDialogButtonBox.StandardButton.Ok)
+        self.bb.button(QDialogButtonBox.StandardButton.Yes).setDefault(True)
         self.title_label = title = QLabel('A dummy title')
         f = title.font()
         f.setBold(True)
@@ -150,10 +151,10 @@ class ProceedQuestion(QWidget):
         l.addWidget(self.bb)
 
         self.ask_question.connect(self.do_ask_question,
-                type=Qt.QueuedConnection)
-        self.setFocusPolicy(Qt.NoFocus)
+                type=Qt.ConnectionType.QueuedConnection)
+        self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         for child in self.findChildren(QWidget):
-            child.setFocusPolicy(Qt.NoFocus)
+            child.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         self.setFocusProxy(self.parent())
         self.resize_timer = t = QTimer(self)
         t.setSingleShot(True), t.setInterval(100), t.timeout.connect(self.parent_resized)
@@ -170,9 +171,9 @@ class ProceedQuestion(QWidget):
     def copy_to_clipboard(self, *args):
         QApplication.clipboard().setText(
                 'calibre, version %s\n%s: %s\n\n%s' %
-                (__version__, unicode(self.windowTitle()),
-                    unicode(self.msg_label.text()),
-                    unicode(self.det_msg.toPlainText())))
+                (__version__, unicode_type(self.windowTitle()),
+                    unicode_type(self.msg_label.text()),
+                    unicode_type(self.det_msg.toPlainText())))
         self.copy_button.setText(_('Copied'))
 
     def action_clicked(self):
@@ -210,7 +211,7 @@ class ProceedQuestion(QWidget):
         self.show_question()
 
     def toggle_det_msg(self, *args):
-        vis = unicode(self.det_msg_toggle.text()) == self.hide_det_msg
+        vis = unicode_type(self.det_msg_toggle.text()) == self.hide_det_msg
         self.det_msg_toggle.setText(self.show_det_msg if vis else
                 self.hide_det_msg)
         self.det_msg.setVisible(not vis)
@@ -240,7 +241,7 @@ class ProceedQuestion(QWidget):
                     QIcon() if question.action_icon is None else question.action_icon)
             # Force the button box to relayout its buttons, as button text
             # might have changed
-            self.bb.setOrientation(Qt.Vertical), self.bb.setOrientation(Qt.Horizontal)
+            self.bb.setOrientation(Qt.Orientation.Vertical), self.bb.setOrientation(Qt.Orientation.Horizontal)
             self.det_msg.setPlainText(question.det_msg or '')
             self.det_msg.setVisible(False)
             self.det_msg_toggle.setVisible(bool(question.det_msg))
@@ -249,15 +250,15 @@ class ProceedQuestion(QWidget):
             if question.checkbox_msg is not None:
                 self.checkbox.setText(question.checkbox_msg)
                 self.checkbox.setChecked(question.checkbox_checked)
-            self.bb.button(self.bb.Ok).setVisible(question.show_ok)
-            self.bb.button(self.bb.Yes).setVisible(not question.show_ok)
-            self.bb.button(self.bb.No).setVisible(not question.show_ok)
+            self.bb.button(QDialogButtonBox.StandardButton.Ok).setVisible(question.show_ok)
+            self.bb.button(QDialogButtonBox.StandardButton.Yes).setVisible(not question.show_ok)
+            self.bb.button(QDialogButtonBox.StandardButton.No).setVisible(not question.show_ok)
             self.copy_button.setVisible(bool(question.show_copy_button))
             self.action_button.setVisible(question.action_callback is not None)
             self.toggle_det_msg() if question.show_det else self.do_resize()
             self.show_widget()
             button = self.action_button if question.focus_action and question.action_callback is not None else \
-                (self.bb.button(self.bb.Ok) if question.show_ok else self.bb.button(self.bb.Yes))
+                (self.bb.button(QDialogButtonBox.StandardButton.Ok) if question.show_ok else self.bb.button(QDialogButtonBox.StandardButton.Yes))
             button.setDefault(True)
             self.raise_()
             self.start_show_animation()
@@ -267,9 +268,16 @@ class ProceedQuestion(QWidget):
             return
 
         dpr = getattr(self, 'devicePixelRatioF', self.devicePixelRatio)()
-        p = QImage(dpr * self.size(), QImage.Format_ARGB32_Premultiplied)
+        p = QImage(dpr * self.size(), QImage.Format.Format_ARGB32_Premultiplied)
         p.setDevicePixelRatio(dpr)
-        self.render(p)
+        # For some reason, Qt scrolls the book view when rendering this widget,
+        # for the very first time, so manually preserve its position
+        pr = getattr(self.parent(), 'library_view', None)
+        if not hasattr(pr, 'preserve_state'):
+            self.render(p)
+        else:
+            with pr.preserve_state():
+                self.render(p)
         self.rendered_pixmap = QPixmap.fromImage(p)
         self.original_visibility = v = []
         for child in self.findChildren(QWidget):
@@ -369,8 +377,8 @@ class ProceedQuestion(QWidget):
 
     def paintEvent(self, ev):
         painter = QPainter(self)
-        painter.setRenderHint(QPainter.Antialiasing, True)
-        painter.setRenderHint(QPainter.SmoothPixmapTransform, True)
+        painter.setRenderHint(QPainter.RenderHint.Antialiasing, True)
+        painter.setRenderHint(QPainter.RenderHint.SmoothPixmapTransform, True)
         try:
             if self.rendered_pixmap is None:
                 self.paint_background(painter)
@@ -387,13 +395,13 @@ class ProceedQuestion(QWidget):
         br = 12  # border_radius
         bw = 1  # border_width
         pal = self.palette()
-        c = pal.color(pal.Window)
+        c = pal.color(QPalette.ColorRole.Window)
         c.setAlphaF(0.9)
         p = QPainterPath()
         p.addRoundedRect(QRectF(self.rect()), br, br)
         painter.fillPath(p, c)
         p.addRoundedRect(QRectF(self.rect()).adjusted(bw, bw, -bw, -bw), br, br)
-        painter.fillPath(p, pal.color(pal.WindowText))
+        painter.fillPath(p, pal.color(QPalette.ColorRole.WindowText))
 
 
 def main():
@@ -417,5 +425,6 @@ def main():
     QTimer.singleShot(10, doit)
     app.exec_()
 
+
 if __name__ == '__main__':
     main()
