@@ -6,13 +6,13 @@ __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, re
+import os, re, traceback
 from functools import partial
 
 from PyQt5.Qt import (
     QStyledItemDelegate, Qt, QTreeView, pyqtSignal, QSize, QIcon, QApplication,
     QMenu, QPoint, QToolTip, QCursor, QDrag, QRect, QModelIndex,
-    QLinearGradient, QPalette, QColor, QPen, QBrush, QFont
+    QLinearGradient, QPalette, QColor, QPen, QBrush, QFont, QTimer
 )
 
 from calibre import sanitize_file_name
@@ -22,7 +22,8 @@ from calibre.gui2.complete2 import EditWithComplete
 from calibre.gui2.tag_browser.model import (TagTreeItem, TAG_SEARCH_STATES,
         TagsModel, DRAG_IMAGE_ROLE, COUNT_ROLE)
 from calibre.gui2.widgets import EnLineEdit
-from calibre.gui2 import config, gprefs, choose_files, pixmap_to_data, rating_font, empty_index
+from calibre.gui2 import (config, gprefs, choose_files, pixmap_to_data,
+                          rating_font, empty_index, question_dialog)
 from calibre.utils.icu import sort_key
 from calibre.utils.serialize import json_loads
 from polyglot.builtins import unicode_type, range, zip
@@ -30,12 +31,13 @@ from polyglot.builtins import unicode_type, range, zip
 
 class TagDelegate(QStyledItemDelegate):  # {{{
 
-    def __init__(self, *args, **kwargs):
-        QStyledItemDelegate.__init__(self, *args, **kwargs)
+    def __init__(self, tags_view):
+        QStyledItemDelegate.__init__(self, tags_view)
         self.old_look = False
         self.rating_pat = re.compile(r'[%s]' % rating_to_stars(3, True))
         self.rating_font = QFont(rating_font())
         self.completion_data = None
+        self.tags_view = tags_view
 
     def draw_average_rating(self, item, style, painter, option, widget):
         rating = item.average_rating
@@ -66,7 +68,7 @@ class TagDelegate(QStyledItemDelegate):  # {{{
         if set_color:
             painter.save()
             pen = painter.pen()
-            pen.setColor(QColor(Qt.black))
+            pen.setColor(QColor(Qt.GlobalColor.black))
             painter.setPen(pen)
         painter.drawText(rect, flags, text)
         if set_color:
@@ -74,7 +76,7 @@ class TagDelegate(QStyledItemDelegate):  # {{{
 
     def draw_text(self, style, painter, option, widget, index, item):
         tr = style.subElementRect(style.SE_ItemViewItemText, option, widget)
-        text = index.data(Qt.DisplayRole)
+        text = index.data(Qt.ItemDataRole.DisplayRole)
         hover = option.state & style.State_MouseOver
         is_search = (True if item.type == TagTreeItem.TAG and
                             item.tag.category == 'search' else False)
@@ -83,20 +85,20 @@ class TagDelegate(QStyledItemDelegate):  # {{{
             width = painter.fontMetrics().boundingRect(count).width()
             r = QRect(tr)
             r.setRight(r.right() - 1), r.setLeft(r.right() - width - 4)
-            self.paint_text(painter, r, Qt.AlignCenter | Qt.TextSingleLine, count, hover)
+            self.paint_text(painter, r, Qt.AlignmentFlag.AlignCenter | Qt.TextFlag.TextSingleLine, count, hover)
             tr.setRight(r.left() - 1)
         else:
             tr.setRight(tr.right() - 1)
         is_rating = item.type == TagTreeItem.TAG and not self.rating_pat.sub('', text)
         if is_rating:
             painter.setFont(self.rating_font)
-        flags = Qt.AlignVCenter | Qt.AlignLeft | Qt.TextSingleLine
+        flags = Qt.AlignmentFlag.AlignVCenter | Qt.AlignmentFlag.AlignLeft | Qt.TextFlag.TextSingleLine
         lr = QRect(tr)
         lr.setRight(lr.right() * 2)
         br = painter.boundingRect(lr, flags, text)
         if br.width() > tr.width():
             g = QLinearGradient(tr.topLeft(), tr.topRight())
-            c = option.palette.color(QPalette.WindowText)
+            c = option.palette.color(QPalette.ColorRole.WindowText)
             g.setColorAt(0, c), g.setColorAt(0.8, c)
             c = QColor(c)
             c.setAlpha(0)
@@ -111,7 +113,7 @@ class TagDelegate(QStyledItemDelegate):  # {{{
         widget = self.parent()
         style = QApplication.style() if widget is None else widget.style()
         self.initStyleOption(option, index)
-        item = index.data(Qt.UserRole)
+        item = index.data(Qt.ItemDataRole.UserRole)
         self.draw_icon(style, painter, option, widget)
         painter.save()
         self.draw_text(style, painter, option, widget, index, item)
@@ -127,6 +129,27 @@ class TagDelegate(QStyledItemDelegate):  # {{{
         self.completion_data = data
 
     def createEditor(self, parent, option, index):
+        item = self.tags_view.model().get_node(index)
+        if not item.ignore_vl:
+            if item.use_vl is None:
+                if self.tags_view.model().get_in_vl():
+                    item.use_vl = question_dialog(self.tags_view,
+                                      _('Rename in Virtual library'), '<p>' +
+                                      _('Do you want this rename to apply only to books '
+                                        'in the current Virtual library?') + '</p>',
+                                      yes_text=_('Yes, apply only in VL'),
+                                      no_text=_('No, apply in entire library'))
+                else:
+                    item.use_vl = False
+            elif not item.use_vl and self.tags_view.model().get_in_vl():
+                item.use_vl = not question_dialog(self.tags_view,
+                                    _('Rename in Virtual library'), '<p>' +
+                                    _('A virtual library is active but you are renaming '
+                                      'the item in all books in your library. Is '
+                                      'this really what you want to do?') + '</p>',
+                                    yes_text=_('Yes, apply in entire library'),
+                                    no_text=_('No, apply only in VL'),
+                                    skip_dialog_name='tag_item_rename_in_entire_library')
         if self.completion_data:
             editor = EditWithComplete(parent)
             editor.set_separator(None)
@@ -157,9 +180,11 @@ class TagsView(QTreeView):  # {{{
     restriction_error       = pyqtSignal()
     tag_item_delete         = pyqtSignal(object, object, object, object, object)
     apply_tag_to_selected   = pyqtSignal(object, object, object)
+    edit_enum_values        = pyqtSignal(object, object, object)
 
     def __init__(self, parent=None):
         QTreeView.__init__(self, parent=None)
+        self.setProperty('frame_for_focus', True)
         self.setMouseTracking(True)
         self.alter_tb = None
         self.disable_recounting = False
@@ -168,7 +193,7 @@ class TagsView(QTreeView):  # {{{
         self.setTabKeyNavigation(True)
         self.setAnimated(True)
         self.setHeaderHidden(True)
-        self.setItemDelegate(TagDelegate(self))
+        self.setItemDelegate(TagDelegate(tags_view=self))
         self.made_connections = False
         self.setAcceptDrops(True)
         self.setDragEnabled(True)
@@ -182,21 +207,20 @@ class TagsView(QTreeView):  # {{{
         self.edit_metadata_icon = QIcon(I('edit_input.png'))
         self.delete_icon = QIcon(I('list_remove.png'))
         self.rename_icon = QIcon(I('edit-undo.png'))
+        self.plus_icon = QIcon(I('plus.png'))
+        self.minus_icon = QIcon(I('minus.png'))
 
         self._model = TagsModel(self)
         self._model.search_item_renamed.connect(self.search_item_renamed)
         self._model.refresh_required.connect(self.refresh_required,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self._model.tag_item_renamed.connect(self.tag_item_renamed)
         self._model.restriction_error.connect(self.restriction_error)
         self._model.user_categories_edited.connect(self.user_categories_edited,
-                type=Qt.QueuedConnection)
+                type=Qt.ConnectionType.QueuedConnection)
         self._model.drag_drop_finished.connect(self.drag_drop_finished)
-        self.set_look_and_feel()
-        # Allowing keyboard focus looks bad in the Qt Fusion style and is useless
-        # anyway since the enter/spacebar keys do nothing
-        self.setFocusPolicy(Qt.NoFocus)
-        QApplication.instance().palette_changed.connect(self.set_style_sheet, type=Qt.QueuedConnection)
+        self.set_look_and_feel(first=True)
+        QApplication.instance().palette_changed.connect(self.set_style_sheet, type=Qt.ConnectionType.QueuedConnection)
 
     def set_style_sheet(self):
         stylish_tb = '''
@@ -221,11 +245,27 @@ class TagsView(QTreeView):  # {{{
         '''.replace('PAD', unicode_type(gprefs['tag_browser_item_padding'])) + (
             '' if gprefs['tag_browser_old_look'] else stylish_tb))
 
-    def set_look_and_feel(self):
+    def set_look_and_feel(self, first=False):
         self.set_style_sheet()
         self.setAlternatingRowColors(gprefs['tag_browser_old_look'])
         self.itemDelegate().old_look = gprefs['tag_browser_old_look']
 
+        if gprefs['tag_browser_allow_keyboard_focus']:
+            self.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
+        else:
+            self.setFocusPolicy(Qt.FocusPolicy.NoFocus)
+        # Ensure the TB doesn't keep the focus it might already have. When this
+        # method is first called during GUI initialization not everything is
+        # set up, in which case don't try to change the focus.
+        # Note: this process has the side effect of moving the focus to the
+        # library view whenever a look & feel preference is changed.
+        if not first:
+            try:
+                from calibre.gui2.ui import get_gui
+                get_gui().shift_esc()
+            except:
+                traceback.print_exc()
+
     @property
     def hidden_categories(self):
         return self._model.hidden_categories
@@ -271,7 +311,7 @@ class TagsView(QTreeView):  # {{{
         self.alter_tb = alter_tb
         self.pane_is_visible = True  # because TagsModel.set_database did a recount
         self.setModel(self._model)
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         pop = self.db.CATEGORY_SORTS.index(config['sort_tags_by'])
         self.alter_tb.sort_menu.actions()[pop].setChecked(True)
         try:
@@ -282,13 +322,38 @@ class TagsView(QTreeView):  # {{{
         if not self.made_connections:
             self.clicked.connect(self.toggle)
             self.customContextMenuRequested.connect(self.show_context_menu)
-            self.refresh_required.connect(self.recount, type=Qt.QueuedConnection)
+            self.refresh_required.connect(self.recount, type=Qt.ConnectionType.QueuedConnection)
             self.alter_tb.sort_menu.triggered.connect(self.sort_changed)
             self.alter_tb.match_menu.triggered.connect(self.match_changed)
             self.made_connections = True
         self.refresh_signal_processed = True
         db.add_listener(self.database_changed)
         self.expanded.connect(self.item_expanded)
+        self.collapsed.connect(self.collapse_node_and_children)
+
+    def keyPressEvent(self, event):
+        # I don't see how current_index can ever be not valid, but ...
+        if self.currentIndex().isValid():
+            if (gprefs['tag_browser_allow_keyboard_focus'] and
+                    event.key() == Qt.Key.Key_Return and self.state() != self.EditingState):
+                self.toggle_current_index()
+                return
+            # If this is an edit request, mark the node to request whether to use VLs
+            # As far as I can tell, F2 is used across all platforms
+            if event.key() == Qt.Key.Key_F2:
+                node = self.model().get_node(self.currentIndex())
+                if node.type == TagTreeItem.TAG:
+                    # Saved search nodes don't use the VL test/dialog
+                    node.use_vl = None
+                    node.ignore_vl = node.tag.category == 'search'
+                else:
+                    # Don't open the editor for non-editable items
+                    if not node.category_key.startswith('@') or node.is_gst:
+                        return
+                    # Category nodes don't use the VL test/dialog
+                    node.use_vl = False
+                    node.ignore_vl = True
+        QTreeView.keyPressEvent(self, event)
 
     def database_changed(self, event, ids):
         if self.refresh_signal_processed:
@@ -322,17 +387,17 @@ class TagsView(QTreeView):  # {{{
             pass
 
     def mousePressEvent(self, event):
-        if event.buttons() & Qt.LeftButton:
+        if event.buttons() & Qt.MouseButton.LeftButton:
             self.possible_drag_start = event.pos()
         return QTreeView.mousePressEvent(self, event)
 
     def mouseMoveEvent(self, event):
         dex = self.indexAt(event.pos())
         if dex.isValid():
-            self.setCursor(Qt.PointingHandCursor)
+            self.setCursor(Qt.CursorShape.PointingHandCursor)
         else:
             self.unsetCursor()
-        if not event.buttons() & Qt.LeftButton:
+        if not event.buttons() & Qt.MouseButton.LeftButton:
             return
         if not dex.isValid():
             QTreeView.mouseMoveEvent(self, event)
@@ -343,7 +408,7 @@ class TagsView(QTreeView):  # {{{
             QTreeView.mouseMoveEvent(self, event)
             return
 
-        if not self._model.flags(dex) & Qt.ItemIsDragEnabled:
+        if not self._model.flags(dex) & Qt.ItemFlag.ItemIsDragEnabled:
             QTreeView.mouseMoveEvent(self, event)
             return
         md = self._model.mimeData([dex])
@@ -359,9 +424,9 @@ class TagsView(QTreeView):  # {{{
             categories stops working. Don't know why. To avoid the problem
             we fix the action in dragMoveEvent.
             '''
-            drag.exec_(Qt.CopyAction|Qt.MoveAction, Qt.CopyAction)
+            drag.exec_(Qt.DropAction.CopyAction|Qt.DropAction.MoveAction, Qt.DropAction.CopyAction)
         else:
-            drag.exec_(Qt.CopyAction)
+            drag.exec_(Qt.DropAction.CopyAction)
 
     def mouseDoubleClickEvent(self, event):
         # swallow these to avoid toggling and editing at the same time
@@ -387,9 +452,17 @@ class TagsView(QTreeView):  # {{{
         in TAG_SEARCH_STATES
         '''
         modifiers = int(QApplication.keyboardModifiers())
-        exclusive = modifiers not in (Qt.CTRL, Qt.SHIFT)
+        exclusive = modifiers not in (Qt.Modifier.CTRL, Qt.Modifier.SHIFT)
         if self._model.toggle(index, exclusive, set_to=set_to):
+            # Reset the focus back to TB if it has it before the toggle
+            # Must ask this question before starting the search because
+            # it changes the focus
+            has_focus = self.hasFocus()
             self.tags_marked.emit(self.search_string)
+            if has_focus and gprefs['tag_browser_allow_keyboard_focus']:
+                # Reset the focus to the TB. Use the singleshot in case
+                # some of searching is done using queued signals.
+                QTimer.singleShot(0, lambda: self.setFocus())
 
     def conditional_clear(self, search_string):
         if search_string != self.search_string:
@@ -397,7 +470,7 @@ class TagsView(QTreeView):  # {{{
 
     def context_menu_handler(self, action=None, category=None,
                              key=None, index=None, search_state=None,
-                             use_vl=None, is_first_letter=False):
+                             is_first_letter=False, ignore_vl=False):
         if not action:
             return
         try:
@@ -419,7 +492,6 @@ class TagsView(QTreeView):  # {{{
                         self._model.set_custom_category_icon(key, unicode_type(path))
                         self.recount()
                 except:
-                    import traceback
                     traceback.print_exc()
                 return
             if action == 'clear_icon':
@@ -437,12 +509,14 @@ class TagsView(QTreeView):  # {{{
             if action == 'edit_item_no_vl':
                 item = self.model().get_node(index)
                 item.use_vl = False
+                item.ignore_vl = ignore_vl
                 set_completion_data(category)
                 self.edit(index)
                 return
             if action == 'edit_item_in_vl':
                 item = self.model().get_node(index)
                 item.use_vl = True
+                item.ignore_vl = ignore_vl
                 set_completion_data(category)
                 self.edit(index)
                 return
@@ -540,12 +614,14 @@ class TagsView(QTreeView):  # {{{
                 if item is not None:
                     self.apply_to_selected_books(item, True)
                 return
+            elif action == 'edit_enum':
+                self.edit_enum_values.emit(self, self.db, key)
+                return
             self.db.new_api.set_pref('tag_browser_hidden_categories', list(self.hidden_categories))
             if reset_filter_categories:
                 self._model.set_categories_filter(None)
             self._model.rebuild_node_tree()
         except Exception:
-            import traceback
             traceback.print_exc()
             return
 
@@ -573,9 +649,24 @@ class TagsView(QTreeView):  # {{{
 
         index = self.indexAt(point)
         self.context_menu = QMenu(self)
+        added_show_hidden_categories = False
 
+        def add_show_hidden_categories():
+            nonlocal added_show_hidden_categories
+            if self.hidden_categories and not added_show_hidden_categories:
+                added_show_hidden_categories = True
+                m = self.context_menu.addMenu(_('Show category'))
+                for col in sorted(self.hidden_categories,
+                        key=lambda x: sort_key(self.db.field_metadata[x]['name'])):
+                    m.addAction(self.db.field_metadata[col]['name'],
+                        partial(self.context_menu_handler, action='show', category=col))
+                m.addSeparator()
+                m.addAction(_('All categories'),
+                        partial(self.context_menu_handler, action='defaults'))
+
+        search_submenu = None
         if index.isValid():
-            item = index.data(Qt.UserRole)
+            item = index.data(Qt.ItemDataRole.UserRole)
             tag = None
             tag_item = item
 
@@ -601,15 +692,16 @@ class TagsView(QTreeView):  # {{{
                     # the possibility of renaming that item.
                     if tag.is_editable or tag.is_hierarchical:
                         # Add the 'rename' items to both interior and leaf nodes
-                        if self.model().get_in_vl():
+                        if fm['datatype'] != 'enumeration':
+                            if self.model().get_in_vl():
+                                self.context_menu.addAction(self.rename_icon,
+                                        _('Rename %s in Virtual library')%display_name(tag),
+                                        partial(self.context_menu_handler, action='edit_item_in_vl',
+                                                index=index, category=key))
                             self.context_menu.addAction(self.rename_icon,
-                                                    _('Rename %s in Virtual library')%display_name(tag),
-                                    partial(self.context_menu_handler, action='edit_item_in_vl',
-                                            index=index, category=key))
-                        self.context_menu.addAction(self.rename_icon,
-                                                _('Rename %s')%display_name(tag),
-                                partial(self.context_menu_handler, action='edit_item_no_vl',
-                                        index=index, category=key))
+                                        _('Rename %s')%display_name(tag),
+                                        partial(self.context_menu_handler, action='edit_item_no_vl',
+                                                index=index, category=key))
                         if key in ('tags', 'series', 'publisher') or \
                                 self._model.db.field_metadata.is_custom_field(key):
                             if self.model().get_in_vl():
@@ -660,11 +752,11 @@ class TagsView(QTreeView):  # {{{
                         # from selected books
                         if fm['datatype'] != 'rating':
                             m = self.context_menu.addMenu(self.edit_metadata_icon,
-                                            _('Apply %s to selected books')%display_name(tag))
-                            m.addAction(QIcon(I('plus.png')),
+                                            _('Add/remove %s to selected books')%display_name(tag))
+                            m.addAction(self.plus_icon,
                                 _('Add %s to selected books') % display_name(tag),
                                 partial(self.context_menu_handler, action='add_tag', index=index))
-                            m.addAction(QIcon(I('minus.png')),
+                            m.addAction(self.minus_icon,
                                 _('Remove %s from selected books') % display_name(tag),
                                 partial(self.context_menu_handler, action='remove_tag', index=index))
 
@@ -672,7 +764,7 @@ class TagsView(QTreeView):  # {{{
                         self.context_menu.addAction(self.rename_icon,
                                                     _('Rename %s')%display_name(tag),
                             partial(self.context_menu_handler, action='edit_item_no_vl',
-                                    index=index))
+                                    index=index, ignore_vl=True))
                         self.context_menu.addAction(self.delete_icon,
                                 _('Delete Saved search %s')%display_name(tag),
                                 partial(self.context_menu_handler,
@@ -686,28 +778,30 @@ class TagsView(QTreeView):  # {{{
                                     key=key, index=tag_item))
                     if tag.is_searchable:
                         # Add the search for value items. All leaf nodes are searchable
-                        self.context_menu.addAction(self.search_icon,
+                        self.context_menu.addSeparator()
+                        search_submenu = self.context_menu.addMenu(_('Search'))
+                        search_submenu.addAction(self.search_icon,
                                 _('Search for %s')%display_name(tag),
                                 partial(self.context_menu_handler, action='search',
                                         search_state=TAG_SEARCH_STATES['mark_plus'],
                                         index=index))
-                        self.context_menu.addAction(self.search_icon,
+                        search_submenu.addAction(self.search_icon,
                                 _('Search for everything but %s')%display_name(tag),
                                 partial(self.context_menu_handler, action='search',
                                         search_state=TAG_SEARCH_STATES['mark_minus'],
                                         index=index))
-                        self.context_menu.addAction(self.search_copy_icon,
-                                _('Search using saved search expression'),
-                                partial(self.context_menu_handler, action='raw_search',
-                                        key=tag.name))
-
+                        if key == 'search':
+                            search_submenu.addAction(self.search_copy_icon,
+                                     _('Search using saved search expression'),
+                                     partial(self.context_menu_handler, action='raw_search',
+                                             key=tag.name))
                     self.context_menu.addSeparator()
                 elif key.startswith('@') and not item.is_gst:
                     if item.can_be_edited:
                         self.context_menu.addAction(self.rename_icon,
                             _('Rename %s')%item.py_name,
                             partial(self.context_menu_handler, action='edit_item_no_vl',
-                                    index=index))
+                                    index=index, ignore_vl=True))
                     self.context_menu.addAction(self.user_category_icon,
                             _('Add sub-category to %s')%item.py_name,
                             partial(self.context_menu_handler,
@@ -717,36 +811,29 @@ class TagsView(QTreeView):  # {{{
                             partial(self.context_menu_handler,
                                     action='delete_user_category', key=key))
                     self.context_menu.addSeparator()
-                # Hide/Show/Restore categories
-                self.context_menu.addAction(_('Hide category %s') % category,
-                    partial(self.context_menu_handler, action='hide',
-                            category=key))
-                if self.hidden_categories:
-                    m = self.context_menu.addMenu(_('Show category'))
-                    for col in sorted(self.hidden_categories,
-                            key=lambda x: sort_key(self.db.field_metadata[x]['name'])):
-                        m.addAction(self.db.field_metadata[col]['name'],
-                            partial(self.context_menu_handler, action='show', category=col))
-
                 # search by category. Some categories are not searchable, such
                 # as search and news
                 if item.tag.is_searchable:
-                    self.context_menu.addAction(self.search_icon,
+                    if search_submenu is None:
+                        search_submenu = self.context_menu.addMenu(_('Search'))
+                        self.context_menu.addSeparator()
+                    search_submenu.addAction(self.search_icon,
                             _('Search for books in category %s')%category,
                             partial(self.context_menu_handler,
                                     action='search_category',
                                     index=self._model.createIndex(item.row(), 0, item),
                                     search_state=TAG_SEARCH_STATES['mark_plus']))
-                    self.context_menu.addAction(self.search_icon,
+                    search_submenu.addAction(self.search_icon,
                             _('Search for books not in category %s')%category,
                             partial(self.context_menu_handler,
                                     action='search_category',
                                     index=self._model.createIndex(item.row(), 0, item),
                                     search_state=TAG_SEARCH_STATES['mark_minus']))
+
                 # Offer specific editors for tags/series/publishers/saved searches
                 self.context_menu.addSeparator()
                 if key in ['tags', 'publisher', 'series'] or (
-                        self.db.field_metadata[key]['is_custom'] and self.db.field_metadata[key]['datatype'] != 'composite'):
+                        fm['is_custom'] and fm['datatype'] != 'composite'):
                     if tag_item.type == TagTreeItem.CATEGORY and tag_item.temporary:
                         self.context_menu.addAction(_('Manage %s')%category,
                             partial(self.context_menu_handler, action='open_editor',
@@ -757,6 +844,10 @@ class TagsView(QTreeView):  # {{{
                             partial(self.context_menu_handler, action='open_editor',
                                     category=tag.original_name if tag else None,
                                     key=key))
+                    if fm['datatype'] == 'enumeration':
+                        self.context_menu.addAction(_('Edit permissible values for %s')%category,
+                            partial(self.context_menu_handler, action='edit_enum',
+                                    key=key))
                 elif key == 'authors':
                     if tag_item.type == TagTreeItem.CATEGORY:
                         if tag_item.temporary:
@@ -775,6 +866,13 @@ class TagsView(QTreeView):  # {{{
                         partial(self.context_menu_handler, action='manage_searches',
                                 category=tag.name if tag else None))
 
+                # Hide/Show/Restore categories
+                self.context_menu.addSeparator()
+                self.context_menu.addAction(_('Hide category %s') % category,
+                    partial(self.context_menu_handler, action='hide',
+                            category=key))
+                add_show_hidden_categories()
+
                 if tag is None:
                     self.context_menu.addSeparator()
                     self.context_menu.addAction(_('Change category icon'),
@@ -786,19 +884,19 @@ class TagsView(QTreeView):  # {{{
                 self.context_menu.addSeparator()
                 if key.startswith('@') and \
                         key[1:] in self.db.new_api.pref('user_categories', {}).keys():
-                    self.context_menu.addAction(_('Manage User categories'),
+                    self.context_menu.addAction(self.user_category_icon,
+                            _('Manage User categories'),
                             partial(self.context_menu_handler, action='manage_categories',
                                     category=key[1:]))
                 else:
-                    self.context_menu.addAction(_('Manage User categories'),
+                    self.context_menu.addAction(self.user_category_icon,
+                            _('Manage User categories'),
                             partial(self.context_menu_handler, action='manage_categories',
                                     category=None))
-
         if self.hidden_categories:
             if not self.context_menu.isEmpty():
                 self.context_menu.addSeparator()
-            self.context_menu.addAction(_('Show all categories'),
-                        partial(self.context_menu_handler, action='defaults'))
+            add_show_hidden_categories()
 
         m = self.context_menu.addMenu(_('Change sub-categorization scheme'))
         da = m.addAction(_('Disable'),
@@ -826,22 +924,78 @@ class TagsView(QTreeView):  # {{{
             da.setToolTip('*')
             pa.setToolTip('*')
 
-        if index.isValid() and self.model().rowCount(index) > 0:
-            self.context_menu.addSeparator()
-            self.context_menu.addAction(_('E&xpand all children'), partial(self.expand_node_and_descendants, index))
+        # Add expand menu items
+        self.context_menu.addSeparator()
+        m = self.context_menu.addMenu(_('Expand or collapse'))
+        try:
+            node_name = self._model.get_node(index).tag.name
+        except AttributeError:
+            pass
+        else:
+            if self.has_children(index) and not self.isExpanded(index):
+                m.addAction(self.plus_icon,
+                            _('Expand {0}').format(node_name), partial(self.expand, index))
+            if self.has_unexpanded_children(index):
+                m.addAction(self.plus_icon,
+                            _('Expand {0} and its children').format(node_name),
+                                            partial(self.expand_node_and_children, index))
 
-        self.context_menu.addAction(_('Collapse all levels'), self.collapseAll)
+        # Add menu items to collapse parent nodes
+        idx = index
+        paths = []
+        while True:
+            # First walk up the node tree getting the displayed names of
+            # expanded parent nodes
+            node = self._model.get_node(idx)
+            if node.type == TagTreeItem.ROOT:
+                break
+            if self.has_children(idx) and self.isExpanded(idx):
+                # leaf nodes don't have children so can't be expanded.
+                # Also the leaf node might be collapsed
+                paths.append((node.tag.name, idx))
+            idx = self._model.parent(idx)
+        for p in paths:
+            # Now add the menu items
+            m.addAction(self.minus_icon,
+                        _("Collapse {0}").format(p[0]), partial(self.collapse_node, p[1]))
+        m.addAction(self.minus_icon, _('Collapse all'), self.collapseAll)
 
         if not self.context_menu.isEmpty():
             self.context_menu.popup(self.mapToGlobal(point))
         return True
 
-    def expand_node_and_descendants(self, index):
+    def has_children(self, idx):
+        return self.model().rowCount(idx) > 0
+
+    def collapse_node_and_children(self, idx):
+        self.collapse(idx)
+        for r in range(self.model().rowCount(idx)):
+            self.collapse_node_and_children(idx.child(r, 0))
+
+    def collapse_node(self, idx):
+        if not idx.isValid():
+            return
+        self.collapse_node_and_children(idx)
+        self.setCurrentIndex(idx)
+        self.scrollTo(idx)
+
+    def expand_node_and_children(self, index):
         if not index.isValid():
             return
         self.expand(index)
         for r in range(self.model().rowCount(index)):
-            self.expand_node_and_descendants(index.child(r, 0))
+            self.expand_node_and_children(index.child(r, 0))
+
+    def has_unexpanded_children(self, index):
+        if not index.isValid():
+            return False
+        for r in range(self._model.rowCount(index)):
+            dex = index.child(r, 0)
+            if self._model.rowCount(dex) > 0:
+                if not self.isExpanded(dex):
+                    return True
+                return self.has_unexpanded_children(dex)
+        return False
 
     def collapse_menu_hovered(self, action):
         tip = action.toolTip()
@@ -856,7 +1010,7 @@ class TagsView(QTreeView):  # {{{
         if not index.isValid():
             return
         src_is_tb = event.mimeData().hasFormat('application/calibre+from_tag_browser')
-        item = index.data(Qt.UserRole)
+        item = index.data(Qt.ItemDataRole.UserRole)
         if item.type == TagTreeItem.ROOT:
             return
 
@@ -873,14 +1027,14 @@ class TagsView(QTreeView):  # {{{
                     src_item.tag.category == item.tag.category and
                     not item.temporary and
                     self._model.is_key_a_hierarchical_category(src_item.tag.category)):
-                event.setDropAction(Qt.MoveAction)
+                event.setDropAction(Qt.DropAction.MoveAction)
                 self.setDropIndicatorShown(True)
                 return
         # We aren't dropping an item on its own category. Check if the dest is
         # not a user category and can be dropped on. This covers drops from the
         # booklist. It is OK to drop onto virtual nodes
-        if item.type == TagTreeItem.TAG and self._model.flags(index) & Qt.ItemIsDropEnabled:
-            event.setDropAction(Qt.CopyAction)
+        if item.type == TagTreeItem.TAG and self._model.flags(index) & Qt.ItemFlag.ItemIsDropEnabled:
+            event.setDropAction(Qt.DropAction.CopyAction)
             self.setDropIndicatorShown(not src_is_tb)
             return
         # Now see if we are on a user category and the source can be dropped there
@@ -889,7 +1043,7 @@ class TagsView(QTreeView):  # {{{
             if fm_dest['kind'] == 'user':
                 if src_is_tb:
                     # src_md and src_item are initialized above
-                    if event.dropAction() == Qt.MoveAction:
+                    if event.dropAction() == Qt.DropAction.MoveAction:
                         # can move only from user categories
                         if (src_md[0] == TagTreeItem.TAG and
                                  (not src_md[1].startswith('@') or src_md[2])):
@@ -916,7 +1070,7 @@ class TagsView(QTreeView):  # {{{
             self.model().clear_state()
 
     def is_visible(self, idx):
-        item = idx.data(Qt.UserRole)
+        item = idx.data(Qt.ItemDataRole.UserRole)
         if getattr(item, 'type', None) == TagTreeItem.TAG:
             idx = idx.parent()
         return self.isExpanded(idx)
@@ -980,7 +1134,7 @@ class TagsView(QTreeView):  # {{{
 
     def show_item_at_index(self, idx, box=False,
                            position=QTreeView.PositionAtCenter):
-        if idx.isValid() and idx.data(Qt.UserRole) is not self._model.root_item:
+        if idx.isValid() and idx.data(Qt.ItemDataRole.UserRole) is not self._model.root_item:
             self.expand_parent(idx)
             self.setCurrentIndex(idx)
             self.scrollTo(idx, position)
