@@ -1,26 +1,24 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
 
 from collections import OrderedDict
-from functools import partial
 import textwrap
 
 from PyQt5.Qt import (
-    QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QLabel, QListWidget, QIcon,
-    QSize, QComboBox, QLineEdit, QListWidgetItem, QStyledItemDelegate,
-    QStaticText, Qt, QStyle, QToolButton, QInputDialog, QMenu, pyqtSignal
+    QWidget, QVBoxLayout, QHBoxLayout, QPushButton, QLabel, QListWidget, QIcon, QDialog,
+    QSize, QComboBox, QLineEdit, QListWidgetItem, QStyledItemDelegate, QAbstractItemView,
+    QStaticText, Qt, QStyle, QToolButton, QInputDialog, QMenu, pyqtSignal, QPalette, QItemSelectionModel
 )
 
 from calibre.ebooks.metadata.tag_mapper import map_tags, compile_pat
-from calibre.gui2 import error_dialog, elided_text, Application, question_dialog
+from calibre.gui2 import error_dialog, Application, question_dialog
 from calibre.gui2.ui import get_gui
 from calibre.gui2.widgets2 import Dialog
 from calibre.utils.config import JSONConfig
 from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import iteritems, unicode_type, range, filter
 
 tag_maps = JSONConfig('tag-map-rules')
 
@@ -47,6 +45,7 @@ class RuleEdit(QWidget):
                 ('replace', _('Replace')),
                 ('keep', _('Keep')),
                 ('capitalize', _('Capitalize')),
+                ('titlecase', _('Title-case')),
                 ('lower', _('Lower-case')),
                 ('upper', _('Upper-case')),
                 ('split', _('Split')),
@@ -85,14 +84,14 @@ class RuleEdit(QWidget):
         l.addLayout(h)
         self.action = a = QComboBox(self)
         h.addWidget(a)
-        for action, text in self.ACTION_MAP.iteritems():
+        for action, text in iteritems(self.ACTION_MAP):
             a.addItem(text, action)
         a.currentIndexChanged.connect(self.update_state)
         self.la1 = la = QLabel('\xa0' + self.SUBJECT + '\xa0')
         h.addWidget(la)
         self.match_type = q = QComboBox(self)
         h.addWidget(q)
-        for action, text in self.MATCH_TYPE_MAP.iteritems():
+        for action, text in iteritems(self.MATCH_TYPE_MAP):
             q.addItem(text, action)
         q.currentIndexChanged.connect(self.update_state)
         self.la2 = la = QLabel(':\xa0')
@@ -101,7 +100,7 @@ class RuleEdit(QWidget):
         h.addWidget(q)
         self.tag_editor_button = b = QToolButton(self)
         b.setIcon(QIcon(I('chapters.png')))
-        b.setToolTip(_('Edit the list of tags with the tag editor'))
+        b.setToolTip(_('Edit the list of tags with the Tag editor'))
         h.addWidget(b), b.clicked.connect(self.edit_tags)
         b.setVisible(self.can_use_tag_editor)
         self.h2 = h = QHBoxLayout()
@@ -150,12 +149,12 @@ class RuleEdit(QWidget):
 
     def specialise_context_menu(self, menu):
         if self.can_use_tag_editor:
-            menu.addAction(_('Use the tag editor to edit the list of tags'), self.edit_tags)
+            menu.addAction(_('Use the Tag editor to edit the list of tags'), self.edit_tags)
 
     def edit_tags(self):
         from calibre.gui2.dialogs.tag_editor import TagEditor
-        d = TagEditor(self, get_gui().current_db, current_tags=filter(None, [x.strip() for x in self.query.text().split(',')]))
-        if d.exec_() == d.Accepted:
+        d = TagEditor(self, get_gui().current_db, current_tags=list(filter(None, [x.strip() for x in self.query.text().split(',')])))
+        if d.exec_() == QDialog.DialogCode.Accepted:
             self.query.setText(', '.join(d.tags))
 
     @property
@@ -172,14 +171,14 @@ class RuleEdit(QWidget):
     def rule(self, rule):
         def sc(name):
             c = getattr(self, name)
-            idx = c.findData(unicode(rule.get(name, '')))
+            idx = c.findData(unicode_type(rule.get(name, '')))
             if idx < 0:
                 idx = 0
             c.setCurrentIndex(idx)
         sc('action'), sc('match_type')
         ac = self.action.currentData()
-        self.query.setText(intelligent_strip(ac, unicode(rule.get('query', ''))))
-        self.replace.setText(intelligent_strip(ac, unicode(rule.get('replace', ''))))
+        self.query.setText(intelligent_strip(ac, unicode_type(rule.get('query', ''))))
+        self.replace.setText(intelligent_strip(ac, unicode_type(rule.get('replace', ''))))
 
     def validate(self):
         rule = self.rule
@@ -216,7 +215,7 @@ class RuleEditDialog(Dialog):
             Dialog.accept(self)
 
 
-DATA_ROLE = Qt.UserRole
+DATA_ROLE = Qt.ItemDataRole.UserRole
 RENDER_ROLE = DATA_ROLE + 1
 
 
@@ -224,7 +223,7 @@ class RuleItem(QListWidgetItem):
 
     @staticmethod
     def text_from_rule(rule, parent):
-        query = elided_text(rule['query'], font=parent.font(), width=200, pos='right')
+        query = rule['query']
         text = _(
             '<b>{action}</b> the tag, if it <i>{match_type}</i>: <b>{query}</b>').format(
                 action=RuleEdit.ACTION_MAP[rule['action']], match_type=RuleEdit.MATCH_TYPE_MAP[rule['match_type']], query=query)
@@ -232,7 +231,7 @@ class RuleItem(QListWidgetItem):
             text += '<br>' + _('with the tag:') + ' <b>%s</b>' % rule['replace']
         if rule['action'] == 'split':
             text += '<br>' + _('on the character:') + ' <b>%s</b>' % rule['replace']
-        return text
+        return '<div style="white-space: nowrap">' + text + '</div>'
 
     def __init__(self, rule, parent):
         QListWidgetItem.__init__(self, '', parent)
@@ -252,12 +251,12 @@ class Delegate(QStyledItemDelegate):
         if width and width != st.textWidth():
             st.setTextWidth(width)
         br = st.size()
-        return QSize(br.width(), br.height() + self.MARGIN)
+        return QSize(br.width() + self.MARGIN, br.height() + self.MARGIN)
 
     def paint(self, painter, option, index):
         QStyledItemDelegate.paint(self, painter, option, index)
         pal = option.palette
-        color = pal.color(pal.HighlightedText if option.state & QStyle.State_Selected else pal.Text).name()
+        color = pal.color(QPalette.ColorRole.HighlightedText if option.state & QStyle.StateFlag.State_Selected else QPalette.ColorRole.Text).name()
         text = '<div style="color:%s">%s</div>' % (color, index.data(RENDER_ROLE))
         st = QStaticText(text)
         st.setTextWidth(option.rect.width())
@@ -296,7 +295,7 @@ class Rules(QWidget):
         l.addLayout(h)
         self.rule_list = r = QListWidget(self)
         self.delegate = Delegate(self)
-        r.setSelectionMode(r.ExtendedSelection)
+        r.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
         r.setItemDelegate(self.delegate)
         r.doubleClicked.connect(self.edit_rule)
         h.addWidget(r)
@@ -304,7 +303,7 @@ class Rules(QWidget):
         r.viewport().setAcceptDrops(True)
         r.setDropIndicatorShown(True)
         r.setDragDropMode(r.InternalMove)
-        r.setDefaultDropAction(Qt.MoveAction)
+        r.setDefaultDropAction(Qt.DropAction.MoveAction)
         self.l2 = l = QVBoxLayout()
         h.addLayout(l)
         self.up_button = b = QToolButton(self)
@@ -321,7 +320,7 @@ class Rules(QWidget):
 
     def add_rule(self):
         d = self.RuleEditDialogClass(self)
-        if d.exec_() == d.Accepted:
+        if d.exec_() == QDialog.DialogCode.Accepted:
             i = self.RuleItemClass(d.edit_widget.rule, self.rule_list)
             self.rule_list.scrollToItem(i)
             self.changed.emit()
@@ -330,8 +329,8 @@ class Rules(QWidget):
         i = self.rule_list.currentItem()
         if i is not None:
             d = self.RuleEditDialogClass(self)
-            d.edit_widget.rule = i.data(Qt.UserRole)
-            if d.exec_() == d.Accepted:
+            d.edit_widget.rule = i.data(Qt.ItemDataRole.UserRole)
+            if d.exec_() == QDialog.DialogCode.Accepted:
                 rule = d.edit_widget.rule
                 i.setData(DATA_ROLE, rule)
                 i.setData(RENDER_ROLE, self.RuleItemClass.text_from_rule(rule, self.rule_list))
@@ -345,30 +344,35 @@ class Rules(QWidget):
         if changed:
             self.changed.emit()
 
+    def move_selected(self, delta=-1):
+        current_item = self.rule_list.currentItem()
+        items = self.rule_list.selectedItems()
+        if current_item is None or not items or not len(items):
+            return
+        row_map = {id(item): self.rule_list.row(item) for item in items}
+        items.sort(key=lambda item: row_map[id(item)])
+        num = self.rule_list.count()
+        for item in items:
+            row = row_map[id(item)]
+            nrow = (row + delta + num) % num
+            self.rule_list.takeItem(row)
+            self.rule_list.insertItem(nrow, item)
+        sm = self.rule_list.selectionModel()
+        for item in items:
+            sm.select(self.rule_list.indexFromItem(item), QItemSelectionModel.SelectionFlag.Select)
+        sm.setCurrentIndex(self.rule_list.indexFromItem(current_item), QItemSelectionModel.SelectionFlag.Current)
+        self.changed.emit()
+
     def move_up(self):
-        i = self.rule_list.currentItem()
-        if i is not None:
-            row = self.rule_list.row(i)
-            if row > 0:
-                self.rule_list.takeItem(row)
-                self.rule_list.insertItem(row - 1, i)
-                self.rule_list.setCurrentItem(i)
-                self.changed.emit()
+        self.move_selected()
 
     def move_down(self):
-        i = self.rule_list.currentItem()
-        if i is not None:
-            row = self.rule_list.row(i)
-            if row < self.rule_list.count() - 1:
-                self.rule_list.takeItem(row)
-                self.rule_list.insertItem(row + 1, i)
-                self.rule_list.setCurrentItem(i)
-                self.changed.emit()
+        self.move_selected(1)
 
     @property
     def rules(self):
         ans = []
-        for r in xrange(self.rule_list.count()):
+        for r in range(self.rule_list.count()):
             ans.append(self.rule_list.item(r).data(DATA_ROLE))
         return ans
 
@@ -452,12 +456,16 @@ class SaveLoadMixin(object):
     def build_load_menu(self):
         self.load_menu.clear()
         if len(self.PREFS_OBJECT):
-            for name, rules in self.PREFS_OBJECT.iteritems():
-                self.load_menu.addAction(name).triggered.connect(partial(self.load_ruleset, name))
+            for name, rules in iteritems(self.PREFS_OBJECT):
+                ac = self.load_menu.addAction(name)
+                ac.setObjectName(name)
+                connect_lambda(ac.triggered, self, lambda self: self.load_ruleset(self.sender().objectName()))
             self.load_menu.addSeparator()
             m = self.load_menu.addMenu(_('Delete saved rulesets'))
-            for name, rules in self.PREFS_OBJECT.iteritems():
-                m.addAction(name).triggered.connect(partial(self.delete_ruleset, name))
+            for name, rules in iteritems(self.PREFS_OBJECT):
+                ac = m.addAction(name)
+                ac.setObjectName(name)
+                connect_lambda(ac.triggered, self, lambda self: self.delete_ruleset(self.sender().objectName()))
         else:
             self.load_menu.addAction(_('No saved rulesets available'))
 
@@ -515,7 +523,7 @@ if __name__ == '__main__':
     d = RulesDialog()
     d.rules = [
         {'action':'remove', 'query':'moose', 'match_type':'one_of', 'replace':''},
-        {'action':'replace', 'query':'moose', 'match_type':'one_of', 'replace':'xxxx'},
+        {'action':'replace', 'query':'moose,sfdg,sfdg,dfsg,dfgsh,sd,er,erg,egrer,ger,s,fgfsgfsga', 'match_type':'one_of', 'replace':'xxxx'},
         {'action':'split', 'query':'/', 'match_type':'has', 'replace':'/'},
     ]
     d.exec_()
