@@ -1,29 +1,35 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import absolute_import, division, print_function, unicode_literals
+# License: GPLv3 Copyright: 2013, Kovid Goyal <kovid at kovidgoyal.net>
 
-__license__   = 'GPL v3'
-__copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
-__docformat__ = 'restructuredtext en'
 
-import sys, os, textwrap
-from threading import Thread
+import os
+import sys
+import textwrap
 from functools import partial
-from polyglot.builtins import map, unicode_type, range
+from PyQt5.Qt import (
+    QApplication, QCheckBox, QCursor, QDialog, QDialogButtonBox, QFrame, QGridLayout,
+    QIcon, QInputDialog, QItemSelectionModel, QKeySequence, QLabel, QMenu,
+    QPushButton, QScrollArea, QSize, QSizePolicy, QStackedWidget, Qt, QAbstractItemView,
+    QToolButton, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget, pyqtSignal
+)
+from threading import Thread
 
-from PyQt5.Qt import (QPushButton, QFrame, QMenu, QInputDialog, QCheckBox,
-    QDialog, QVBoxLayout, QDialogButtonBox, QSize, QStackedWidget, QWidget,
-    QLabel, Qt, pyqtSignal, QIcon, QTreeWidget, QGridLayout, QTreeWidgetItem,
-    QToolButton, QItemSelectionModel, QCursor, QKeySequence, QSizePolicy)
-
-from calibre.ebooks.oeb.polish.container import get_container, AZW3Container
+from calibre.constants import TOC_DIALOG_APP_UID, islinux, iswindows
+from calibre.ebooks.oeb.polish.container import AZW3Container, get_container
 from calibre.ebooks.oeb.polish.toc import (
-    get_toc, add_id, TOC, commit_toc, from_xpaths, from_links, from_files)
-from calibre.gui2 import Application, error_dialog, gprefs, info_dialog
+    TOC, add_id, commit_toc, from_files, from_links, from_xpaths, get_toc
+)
+from calibre.gui2 import (
+    Application, error_dialog, gprefs, info_dialog, question_dialog, set_app_uid
+)
+from calibre.gui2.convert.xpath_wizard import XPathEdit
 from calibre.gui2.progress_indicator import ProgressIndicator
 from calibre.gui2.toc.location import ItemEdit
-from calibre.gui2.convert.xpath_wizard import XPathEdit
+from calibre.ptempfile import reset_base_dir
+from calibre.utils.lock import ExclusiveFile
 from calibre.utils.logging import GUILog
+from polyglot.builtins import map, range, unicode_type
 
 ICON_SIZE = 24
 
@@ -50,7 +56,7 @@ class XPathDialog(QDialog):  # {{{
             self.widgets.append(xp)
             l.addWidget(xp)
 
-        self.bb = bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+        self.bb = bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
         self.ssb = b = bb.addButton(_('&Save settings'), bb.ActionRole)
@@ -80,7 +86,7 @@ class XPathDialog(QDialog):  # {{{
             if name:
                 saved = self.prefs.get('xpath_toc_settings', {})
                 # in JSON all keys have to be strings
-                saved[name] = {str(i):x for i, x in enumerate(xpaths)}
+                saved[name] = {unicode_type(i):x for i, x in enumerate(xpaths)}
                 self.prefs.set('xpath_toc_settings', saved)
                 self.setup_load_button()
 
@@ -103,7 +109,7 @@ class XPathDialog(QDialog):  # {{{
     def load_settings(self, name):
         saved = self.prefs.get('xpath_toc_settings', {}).get(name, {})
         for i, w in enumerate(self.widgets):
-            txt = saved.get(str(i), '')
+            txt = saved.get(unicode_type(i), '')
             w.edit.setText(txt)
 
     def check(self):
@@ -126,7 +132,7 @@ class XPathDialog(QDialog):  # {{{
 # }}}
 
 
-class ItemView(QFrame):  # {{{
+class ItemView(QStackedWidget):  # {{{
 
     add_new_item = pyqtSignal(object, object)
     delete_item = pyqtSignal()
@@ -138,19 +144,20 @@ class ItemView(QFrame):  # {{{
     flatten_toc = pyqtSignal()
 
     def __init__(self, parent, prefs):
-        QFrame.__init__(self, parent)
+        QStackedWidget.__init__(self, parent)
         self.prefs = prefs
-        self.setFrameShape(QFrame.StyledPanel)
         self.setMinimumWidth(250)
-        self.stack = s = QStackedWidget(self)
-        self.l = l = QVBoxLayout()
-        self.setLayout(l)
-        l.addWidget(s)
         self.root_pane = rp = QWidget(self)
         self.item_pane = ip = QWidget(self)
         self.current_item = None
-        s.addWidget(rp)
-        s.addWidget(ip)
+        sa = QScrollArea(self)
+        sa.setWidgetResizable(True)
+        sa.setWidget(rp)
+        self.addWidget(sa)
+        sa = QScrollArea(self)
+        sa.setWidgetResizable(True)
+        sa.setWidget(ip)
+        self.addWidget(sa)
 
         self.l1 = la = QLabel('<p>'+_(
             'You can edit existing entries in the Table of Contents by clicking them'
@@ -305,15 +312,22 @@ class ItemView(QFrame):  # {{{
         self.w2.setWordWrap(True)
         l.addWidget(la, l.rowCount(), 0, 1, 2)
 
+    def ask_if_duplicates_should_be_removed(self):
+        return not question_dialog(self, _('Remove duplicates'), _(
+            'Should headings with the same text at the same level be included?'),
+            yes_text=_('&Include duplicates'), no_text=_('&Remove duplicates'))
+
     def create_from_major_headings(self):
-        self.create_from_xpath.emit(['//h:h%d'%i for i in range(1, 4)], True)
+        self.create_from_xpath.emit(['//h:h%d'%i for i in range(1, 4)],
+                self.ask_if_duplicates_should_be_removed())
 
     def create_from_all_headings(self):
-        self.create_from_xpath.emit(['//h:h%d'%i for i in range(1, 7)], True)
+        self.create_from_xpath.emit(['//h:h%d'%i for i in range(1, 7)],
+                self.ask_if_duplicates_should_be_removed())
 
     def create_from_user_xpath(self):
         d = XPathDialog(self, self.prefs)
-        if d.exec_() == d.Accepted and d.xpaths:
+        if d.exec_() == QDialog.DialogCode.Accepted and d.xpaths:
             self.create_from_xpath.emit(d.xpaths, d.remove_duplicates_cb.isChecked())
 
     def hide_azw3_warning(self):
@@ -331,20 +345,20 @@ class ItemView(QFrame):  # {{{
     def __call__(self, item):
         if item is None:
             self.current_item = None
-            self.stack.setCurrentIndex(0)
+            self.setCurrentIndex(0)
         else:
             self.current_item = item
-            self.stack.setCurrentIndex(1)
+            self.setCurrentIndex(1)
             self.populate_item_pane()
 
     def populate_item_pane(self):
         item = self.current_item
-        name = unicode_type(item.data(0, Qt.DisplayRole) or '')
+        name = unicode_type(item.data(0, Qt.ItemDataRole.DisplayRole) or '')
         self.item_pane.heading.setText('<h2>%s</h2>'%name)
-        self.icon_label.setPixmap(item.data(0, Qt.DecorationRole
+        self.icon_label.setPixmap(item.data(0, Qt.ItemDataRole.DecorationRole
                                             ).pixmap(32, 32))
         tt = _('This entry points to an existing destination')
-        toc = item.data(0, Qt.UserRole)
+        toc = item.data(0, Qt.ItemDataRole.UserRole)
         if toc.dest_exists is False:
             tt = _('The location this entry points to does not exist')
         elif toc.dest_exists is None:
@@ -358,7 +372,7 @@ class ItemView(QFrame):  # {{{
 # }}}
 
 
-NODE_FLAGS = (Qt.ItemIsDragEnabled|Qt.ItemIsEditable|Qt.ItemIsEnabled|Qt.ItemIsSelectable|Qt.ItemIsDropEnabled)
+NODE_FLAGS = (Qt.ItemFlag.ItemIsDragEnabled|Qt.ItemFlag.ItemIsEditable|Qt.ItemFlag.ItemIsEnabled|Qt.ItemFlag.ItemIsSelectable|Qt.ItemFlag.ItemIsDropEnabled)
 
 
 class TreeWidget(QTreeWidget):  # {{{
@@ -372,19 +386,19 @@ class TreeWidget(QTreeWidget):  # {{{
         self.setHeaderLabel(_('Table of Contents'))
         self.setIconSize(QSize(ICON_SIZE, ICON_SIZE))
         self.setDragEnabled(True)
-        self.setSelectionMode(self.ExtendedSelection)
+        self.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
         self.viewport().setAcceptDrops(True)
         self.setDropIndicatorShown(True)
         self.setDragDropMode(self.InternalMove)
         self.setAutoScroll(True)
         self.setAutoScrollMargin(ICON_SIZE*2)
-        self.setDefaultDropAction(Qt.MoveAction)
+        self.setDefaultDropAction(Qt.DropAction.MoveAction)
         self.setAutoExpandDelay(1000)
         self.setAnimated(True)
         self.setMouseTracking(True)
         self.in_drop_event = False
         self.root = self.invisibleRootItem()
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         self.customContextMenuRequested.connect(self.show_context_menu)
 
     def push_history(self):
@@ -410,7 +424,7 @@ class TreeWidget(QTreeWidget):  # {{{
                 yield gc
 
     def update_status_tip(self, item):
-        c = item.data(0, Qt.UserRole)
+        c = item.data(0, Qt.ItemDataRole.UserRole)
         if c is not None:
             frag = c.frag or ''
             if frag:
@@ -422,10 +436,10 @@ class TreeWidget(QTreeWidget):  # {{{
 
         def serialize_node(node):
             return {
-                'title': node.data(0, Qt.DisplayRole),
-                'toc_node': node.data(0, Qt.UserRole),
-                'icon': node.data(0, Qt.DecorationRole),
-                'tooltip': node.data(0, Qt.ToolTipRole),
+                'title': node.data(0, Qt.ItemDataRole.DisplayRole),
+                'toc_node': node.data(0, Qt.ItemDataRole.UserRole),
+                'icon': node.data(0, Qt.ItemDataRole.DecorationRole),
+                'tooltip': node.data(0, Qt.ItemDataRole.ToolTipRole),
                 'is_selected': node.isSelected(),
                 'is_expanded': node.isExpanded(),
                 'children': list(map(serialize_node, (node.child(i) for i in range(node.childCount())))),
@@ -438,11 +452,11 @@ class TreeWidget(QTreeWidget):  # {{{
 
         def unserialize_node(dict_node, parent):
             n = QTreeWidgetItem(parent)
-            n.setData(0, Qt.DisplayRole, dict_node['title'])
-            n.setData(0, Qt.UserRole, dict_node['toc_node'])
+            n.setData(0, Qt.ItemDataRole.DisplayRole, dict_node['title'])
+            n.setData(0, Qt.ItemDataRole.UserRole, dict_node['toc_node'])
             n.setFlags(NODE_FLAGS)
-            n.setData(0, Qt.DecorationRole, dict_node['icon'])
-            n.setData(0, Qt.ToolTipRole, dict_node['tooltip'])
+            n.setData(0, Qt.ItemDataRole.DecorationRole, dict_node['icon'])
+            n.setData(0, Qt.ItemDataRole.ToolTipRole, dict_node['tooltip'])
             self.update_status_tip(n)
             n.setExpanded(dict_node['is_expanded'])
             n.setSelected(dict_node['is_selected'])
@@ -473,7 +487,7 @@ class TreeWidget(QTreeWidget):  # {{{
         return ans
 
     def highlight_item(self, item):
-        self.setCurrentItem(item, 0, QItemSelectionModel.ClearAndSelect)
+        self.setCurrentItem(item, 0, QItemSelectionModel.SelectionFlag.ClearAndSelect)
         self.scrollToItem(item)
 
     def check_multi_selection(self):
@@ -583,61 +597,61 @@ class TreeWidget(QTreeWidget):  # {{{
         self.push_history()
         from calibre.utils.titlecase import titlecase
         for item in self.selectedItems():
-            t = unicode_type(item.data(0, Qt.DisplayRole) or '')
-            item.setData(0, Qt.DisplayRole, titlecase(t))
+            t = unicode_type(item.data(0, Qt.ItemDataRole.DisplayRole) or '')
+            item.setData(0, Qt.ItemDataRole.DisplayRole, titlecase(t))
 
     def upper_case(self):
         self.push_history()
         for item in self.selectedItems():
-            t = unicode_type(item.data(0, Qt.DisplayRole) or '')
-            item.setData(0, Qt.DisplayRole, icu_upper(t))
+            t = unicode_type(item.data(0, Qt.ItemDataRole.DisplayRole) or '')
+            item.setData(0, Qt.ItemDataRole.DisplayRole, icu_upper(t))
 
     def lower_case(self):
         self.push_history()
         for item in self.selectedItems():
-            t = unicode_type(item.data(0, Qt.DisplayRole) or '')
-            item.setData(0, Qt.DisplayRole, icu_lower(t))
+            t = unicode_type(item.data(0, Qt.ItemDataRole.DisplayRole) or '')
+            item.setData(0, Qt.ItemDataRole.DisplayRole, icu_lower(t))
 
     def swap_case(self):
         self.push_history()
         from calibre.utils.icu import swapcase
         for item in self.selectedItems():
-            t = unicode_type(item.data(0, Qt.DisplayRole) or '')
-            item.setData(0, Qt.DisplayRole, swapcase(t))
+            t = unicode_type(item.data(0, Qt.ItemDataRole.DisplayRole) or '')
+            item.setData(0, Qt.ItemDataRole.DisplayRole, swapcase(t))
 
     def capitalize(self):
         self.push_history()
         from calibre.utils.icu import capitalize
         for item in self.selectedItems():
-            t = unicode_type(item.data(0, Qt.DisplayRole) or '')
-            item.setData(0, Qt.DisplayRole, capitalize(t))
+            t = unicode_type(item.data(0, Qt.ItemDataRole.DisplayRole) or '')
+            item.setData(0, Qt.ItemDataRole.DisplayRole, capitalize(t))
 
     def bulk_rename(self):
         from calibre.gui2.tweak_book.file_list import get_bulk_rename_settings
-        sort_map = {item:i for i, item in enumerate(self.iter_items())}
-        items = sorted(self.selectedItems(), key=lambda x:sort_map.get(x, -1))
+        sort_map = {id(item):i for i, item in enumerate(self.iter_items())}
+        items = sorted(self.selectedItems(), key=lambda x:sort_map.get(id(x), -1))
         settings = get_bulk_rename_settings(self, len(items), prefix=_('Chapter '), msg=_(
             'All selected items will be renamed to the form prefix-number'), sanitize=lambda x:x, leading_zeros=False)
         fmt, num = settings['prefix'], settings['start']
         if fmt is not None and num is not None:
             self.push_history()
             for i, item in enumerate(items):
-                item.setData(0, Qt.DisplayRole, fmt % (num + i))
+                item.setData(0, Qt.ItemDataRole.DisplayRole, fmt % (num + i))
 
     def keyPressEvent(self, ev):
-        if ev.key() == Qt.Key_Left and ev.modifiers() & Qt.CTRL:
+        if ev.key() == Qt.Key.Key_Left and ev.modifiers() & Qt.Modifier.CTRL:
             self.move_left()
             ev.accept()
-        elif ev.key() == Qt.Key_Right and ev.modifiers() & Qt.CTRL:
+        elif ev.key() == Qt.Key.Key_Right and ev.modifiers() & Qt.Modifier.CTRL:
             self.move_right()
             ev.accept()
-        elif ev.key() == Qt.Key_Up and (ev.modifiers() & Qt.CTRL or ev.modifiers() & Qt.ALT):
+        elif ev.key() == Qt.Key.Key_Up and (ev.modifiers() & Qt.Modifier.CTRL or ev.modifiers() & Qt.Modifier.ALT):
             self.move_up()
             ev.accept()
-        elif ev.key() == Qt.Key_Down and (ev.modifiers() & Qt.CTRL or ev.modifiers() & Qt.ALT):
+        elif ev.key() == Qt.Key.Key_Down and (ev.modifiers() & Qt.Modifier.CTRL or ev.modifiers() & Qt.Modifier.ALT):
             self.move_down()
             ev.accept()
-        elif ev.key() in (Qt.Key_Delete, Qt.Key_Backspace):
+        elif ev.key() in (Qt.Key.Key_Delete, Qt.Key.Key_Backspace):
             self.del_items()
             ev.accept()
         else:
@@ -647,29 +661,29 @@ class TreeWidget(QTreeWidget):  # {{{
         item = self.currentItem()
 
         def key(k):
-            sc = unicode_type(QKeySequence(k | Qt.CTRL).toString(QKeySequence.NativeText))
+            sc = unicode_type(QKeySequence(k | Qt.Modifier.CTRL).toString(QKeySequence.SequenceFormat.NativeText))
             return ' [%s]'%sc
 
         if item is not None:
-            m = QMenu()
+            m = QMenu(self)
             m.addAction(QIcon(I('edit_input.png')), _('Change the location this entry points to'), self.edit_item)
             m.addAction(QIcon(I('modified.png')), _('Bulk rename all selected items'), self.bulk_rename)
             m.addAction(QIcon(I('trash.png')), _('Remove all selected items'), self.del_items)
             m.addSeparator()
-            ci = unicode_type(item.data(0, Qt.DisplayRole) or '')
+            ci = unicode_type(item.data(0, Qt.ItemDataRole.DisplayRole) or '')
             p = item.parent() or self.invisibleRootItem()
             idx = p.indexOfChild(item)
             if idx > 0:
-                m.addAction(QIcon(I('arrow-up.png')), (_('Move "%s" up')%ci)+key(Qt.Key_Up), self.move_up)
+                m.addAction(QIcon(I('arrow-up.png')), (_('Move "%s" up')%ci)+key(Qt.Key.Key_Up), self.move_up)
             if idx + 1 < p.childCount():
-                m.addAction(QIcon(I('arrow-down.png')), (_('Move "%s" down')%ci)+key(Qt.Key_Down), self.move_down)
+                m.addAction(QIcon(I('arrow-down.png')), (_('Move "%s" down')%ci)+key(Qt.Key.Key_Down), self.move_down)
             if item.parent() is not None:
-                m.addAction(QIcon(I('back.png')), (_('Unindent "%s"')%ci)+key(Qt.Key_Left), self.move_left)
+                m.addAction(QIcon(I('back.png')), (_('Unindent "%s"')%ci)+key(Qt.Key.Key_Left), self.move_left)
             if idx > 0:
-                m.addAction(QIcon(I('forward.png')), (_('Indent "%s"')%ci)+key(Qt.Key_Right), self.move_right)
+                m.addAction(QIcon(I('forward.png')), (_('Indent "%s"')%ci)+key(Qt.Key.Key_Right), self.move_right)
 
             m.addSeparator()
-            case_menu = QMenu(_('Change case'))
+            case_menu = QMenu(_('Change case'), m)
             case_menu.addAction(_('Upper case'), self.upper_case)
             case_menu.addAction(_('Lower case'), self.lower_case)
             case_menu.addAction(_('Swap case'), self.swap_case)
@@ -737,7 +751,7 @@ class TOCView(QWidget):  # {{{
         l.addWidget(b, col, 1)
         self.default_msg = _('Double click on an entry to change the text')
         self.hl = hl = QLabel(self.default_msg)
-        hl.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored)
+        hl.setSizePolicy(QSizePolicy.Policy.Ignored, QSizePolicy.Policy.Ignored)
         l.addWidget(hl, col, 2, 1, -1)
         self.item_view = i = ItemView(self, self.prefs)
         self.item_view.delete_item.connect(self.delete_current_item)
@@ -762,7 +776,7 @@ class TOCView(QWidget):  # {{{
         return super(TOCView, self).event(e)
 
     def item_title(self, item):
-        return unicode_type(item.data(0, Qt.DisplayRole) or '')
+        return unicode_type(item.data(0, Qt.ItemDataRole.DisplayRole) or '')
 
     def del_items(self):
         self.tocw.del_items()
@@ -817,8 +831,8 @@ class TOCView(QWidget):  # {{{
     def data_changed(self, top_left, bottom_right):
         for r in range(top_left.row(), bottom_right.row()+1):
             idx = self.tocw.model().index(r, 0, top_left.parent())
-            new_title = unicode_type(idx.data(Qt.DisplayRole) or '').strip()
-            toc = idx.data(Qt.UserRole)
+            new_title = unicode_type(idx.data(Qt.ItemDataRole.DisplayRole) or '').strip()
+            toc = idx.data(Qt.ItemDataRole.UserRole)
             if toc is not None:
                 toc.title = new_title or _('(Untitled)')
             item = self.tocw.itemFromIndex(idx)
@@ -835,16 +849,16 @@ class TOCView(QWidget):  # {{{
         return c
 
     def populate_item(self, c, child):
-        c.setData(0, Qt.DisplayRole, child.title or _('(Untitled)'))
-        c.setData(0, Qt.UserRole, child)
+        c.setData(0, Qt.ItemDataRole.DisplayRole, child.title or _('(Untitled)'))
+        c.setData(0, Qt.ItemDataRole.UserRole, child)
         c.setFlags(NODE_FLAGS)
-        c.setData(0, Qt.DecorationRole, self.icon_map[child.dest_exists])
+        c.setData(0, Qt.ItemDataRole.DecorationRole, self.icon_map[child.dest_exists])
         if child.dest_exists is False:
-            c.setData(0, Qt.ToolTipRole, _(
+            c.setData(0, Qt.ItemDataRole.ToolTipRole, _(
                 'The location this entry point to does not exist:\n%s')
                 %child.dest_error)
         else:
-            c.setData(0, Qt.ToolTipRole, None)
+            c.setData(0, Qt.ItemDataRole.ToolTipRole, None)
 
         self.tocw.update_status_tip(c)
 
@@ -866,7 +880,7 @@ class TOCView(QWidget):  # {{{
                 process_item(child, c)
 
         root = self.root = self.tocw.invisibleRootItem()
-        root.setData(0, Qt.UserRole, self.toc)
+        root.setData(0, Qt.ItemDataRole.UserRole, self.toc)
         process_item(self.toc, root)
         self.tocw.model().dataChanged.connect(self.data_changed)
         self.tocw.currentItemChanged.connect(self.current_item_changed)
@@ -884,7 +898,7 @@ class TOCView(QWidget):  # {{{
         if item is None:
             # New entry at root level
             c = self.create_item(self.root, child)
-            self.tocw.setCurrentItem(c, 0, QItemSelectionModel.ClearAndSelect)
+            self.tocw.setCurrentItem(c, 0, QItemSelectionModel.SelectionFlag.ClearAndSelect)
             self.tocw.scrollToItem(c)
         else:
             if where is None:
@@ -900,7 +914,7 @@ class TOCView(QWidget):  # {{{
                     if where == 'after':
                         idx += 1
                 c = self.create_item(parent, child, idx=idx)
-                self.tocw.setCurrentItem(c, 0, QItemSelectionModel.ClearAndSelect)
+                self.tocw.setCurrentItem(c, 0, QItemSelectionModel.SelectionFlag.ClearAndSelect)
                 self.tocw.scrollToItem(c)
 
     def create_toc(self):
@@ -909,8 +923,8 @@ class TOCView(QWidget):  # {{{
         def process_node(parent, toc_parent):
             for i in range(parent.childCount()):
                 item = parent.child(i)
-                title = unicode_type(item.data(0, Qt.DisplayRole) or '').strip()
-                toc = item.data(0, Qt.UserRole)
+                title = unicode_type(item.data(0, Qt.ItemDataRole.DisplayRole) or '').strip()
+                toc = item.data(0, Qt.ItemDataRole.UserRole)
                 dest, frag = toc.dest, toc.frag
                 toc = toc_parent.add(title, dest, frag)
                 process_node(item, toc)
@@ -986,14 +1000,14 @@ class TOCEditor(QDialog):  # {{{
         ll = self.ll = QVBoxLayout()
         lw.setLayout(ll)
         self.pi = pi = ProgressIndicator()
-        pi.setDisplaySize(200)
+        pi.setDisplaySize(QSize(200, 200))
         pi.startAnimation()
-        ll.addWidget(pi, alignment=Qt.AlignHCenter|Qt.AlignCenter)
+        ll.addWidget(pi, alignment=Qt.AlignmentFlag.AlignHCenter|Qt.AlignmentFlag.AlignCenter)
         la = self.wait_label = QLabel(_('Loading %s, please wait...')%t)
         la.setWordWrap(True)
         f = la.font()
         f.setPointSize(20), la.setFont(f)
-        ll.addWidget(la, alignment=Qt.AlignHCenter|Qt.AlignTop)
+        ll.addWidget(la, alignment=Qt.AlignmentFlag.AlignHCenter|Qt.AlignmentFlag.AlignTop)
         self.toc_view = TOCView(self, self.prefs)
         self.toc_view.add_new_item.connect(self.add_new_item)
         self.toc_view.tocw.history_state_changed.connect(self.update_history_buttons)
@@ -1001,7 +1015,7 @@ class TOCEditor(QDialog):  # {{{
         self.item_edit = ItemEdit(self)
         s.addWidget(self.item_edit)
 
-        bb = self.bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+        bb = self.bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
         l.addWidget(bb)
         bb.accepted.connect(self.accept)
         bb.rejected.connect(self.reject)
@@ -1010,13 +1024,14 @@ class TOCEditor(QDialog):  # {{{
         b.setIcon(QIcon(I('edit-undo.png')))
         b.clicked.connect(self.toc_view.undo)
 
-        self.explode_done.connect(self.read_toc, type=Qt.QueuedConnection)
-        self.writing_done.connect(self.really_accept, type=Qt.QueuedConnection)
+        self.explode_done.connect(self.read_toc, type=Qt.ConnectionType.QueuedConnection)
+        self.writing_done.connect(self.really_accept, type=Qt.ConnectionType.QueuedConnection)
 
-        self.resize(950, 630)
+        r = QApplication.desktop().availableGeometry(self)
+        self.resize(r.width() - 100, r.height() - 100)
         geom = self.prefs.get('toc_editor_window_geom', None)
         if geom is not None:
-            self.restoreGeometry(bytes(geom))
+            QApplication.instance().safe_restore_geometry(self, bytes(geom))
         self.stacks.currentChanged.connect(self.update_history_buttons)
         self.update_history_buttons()
 
@@ -1110,10 +1125,30 @@ class TOCEditor(QDialog):  # {{{
 # }}}
 
 
+def main(path=None, title=None):
+    # Ensure we can continue to function if GUI is closed
+    os.environ.pop('CALIBRE_WORKER_TEMP_DIR', None)
+    reset_base_dir()
+    if iswindows:
+        # Ensure that all instances are grouped together in the task bar. This
+        # prevents them from being grouped with viewer/editor process when
+        # launched from within calibre, as both use calibre-parallel.exe
+        set_app_uid(TOC_DIALOG_APP_UID)
+
+    with ExclusiveFile(path + '.lock') as wf:
+        override = 'calibre-gui' if islinux else None
+        app = Application([], override_program_name=override)
+        d = TOCEditor(path, title=title)
+        d.start()
+        ret = 1
+        if d.exec_() == QDialog.DialogCode.Accepted:
+            ret = 0
+        wf.write('{}'.format(ret).encode('ascii'))
+    del d
+    del app
+    raise SystemExit(ret)
+
+
 if __name__ == '__main__':
-    app = Application([], force_calibre_style=True)
-    app
-    d = TOCEditor(sys.argv[-1])
-    d.start()
-    d.exec_()
-    del d  # Needed to prevent sigsegv in exit cleanup
+    main(path=sys.argv[-1], title='test')
+    os.remove(sys.argv[-1] + '.lock')
