@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import with_statement
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -106,12 +106,10 @@ def config(defaults=None):
                 'directory with filenames containing title and author. '
                 'Available controls are: {%(controls)s}')%dict(
                     templ=DEFAULT_SEND_TEMPLATE, controls=', '.join(FORMAT_ARGS)))
-    x('asciiize', default=True,
-            help=_('Normally, calibre will convert all non English characters into English equivalents '
-                'for the file names. '
-                'WARNING: If you turn this off, you may experience errors when '
-                'saving, depending on how well the filesystem you are saving '
-                'to supports unicode.'))
+    x('asciiize', default=False,
+            help=_('Have calibre convert all non English characters into English equivalents '
+                'for the file names. This is useful if saving to a legacy filesystem '
+                'without full support for Unicode filenames.'))
     x('timefmt', default='%b, %Y',
             help=_('The format in which to display dates. %(day)s - day,'
                 ' %(month)s - month, %(mn)s - month number, %(year)s - year. Default is: %(default)s'
@@ -217,7 +215,7 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
     if hasattr(mi, 'last_modified') and hasattr(mi.last_modified, 'timetuple'):
         format_args['last_modified'] = strftime(timefmt, mi.last_modified.timetuple())
 
-    format_args['id'] = str(id)
+    format_args['id'] = unicode_type(id)
     # Now format the custom fields
     custom_metadata = mi.get_all_user_metadata(make_copy=False)
     for key in custom_metadata:
@@ -247,7 +245,7 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
     components = [x.strip() for x in components.split('/')]
     components = [sanitize_func(x) for x in components if x]
     if not components:
-        components = [str(id)]
+        components = [unicode_type(id)]
     if to_lowercase:
         components = [x.lower() for x in components]
     if replace_whitespace:
@@ -360,7 +358,7 @@ def do_save_book_to_disk(db, book_id, mi, plugboards,
         return not formats_written, book_id, mi.title
 
     for fmt in formats:
-        fmt_path = base_path+'.'+str(fmt)
+        fmt_path = base_path+'.'+unicode_type(fmt)
         try:
             db.copy_format_to(book_id, fmt, fmt_path)
             formats_written = True
